# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, List, Optional
from typing_extensions import Literal


class CalculationLineItem(StripeObject):
    OBJECT_NAME: ClassVar[Literal["tax.calculation_line_item"]] = (
        "tax.calculation_line_item"
    )

    class PerformanceLocationDetails(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix, such as "NY" or "TX".
            """

        address: Address
        _inner_class_types = {"address": Address}

    class TaxBreakdown(StripeObject):
        class Jurisdiction(StripeObject):
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            display_name: str
            """
            A human-readable name for the jurisdiction imposing the tax.
            """
            level: Literal["city", "country", "county", "district", "state"]
            """
            Indicates the level of the jurisdiction imposing the tax.
            """
            state: Optional[str]
            """
            [ISO 3166-2 subdivision code](https://en.wikipedia.org/wiki/ISO_3166-2), without country prefix. For example, "NY" for New York, United States.
            """

        class TaxRateDetails(StripeObject):
            display_name: str
            """
            A localized display name for tax type, intended to be human-readable. For example, "Local Sales and Use Tax", "Value-added tax (VAT)", or "Umsatzsteuer (USt.)".
            """
            percentage_decimal: str
            """
            The tax rate percentage as a string. For example, 8.5% is represented as "8.5".
            """
            tax_type: Literal[
                "admissions_tax",
                "amusement_tax",
                "attendance_tax",
                "communications_tax",
                "entertainment_tax",
                "gross_receipts_tax",
                "gst",
                "hospitality_tax",
                "hst",
                "igst",
                "jct",
                "lease_tax",
                "luxury_tax",
                "pst",
                "qst",
                "resort_tax",
                "retail_delivery_fee",
                "rst",
                "sales_tax",
                "service_tax",
                "tourism_tax",
                "vat",
            ]
            """
            The tax type, such as `vat` or `sales_tax`.
            """

        amount: int
        """
        The amount of tax, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
        """
        jurisdiction: Jurisdiction
        sourcing: Literal["destination", "origin", "performance"]
        """
        Indicates whether the jurisdiction was determined by the origin (merchant's address) or destination (customer's address).
        """
        tax_rate_details: Optional[TaxRateDetails]
        """
        Details regarding the rate for this tax. This field will be `null` when the tax is not imposed, for example if the product is exempt from tax.
        """
        taxability_reason: Literal[
            "customer_exempt",
            "not_collecting",
            "not_subject_to_tax",
            "not_supported",
            "portion_product_exempt",
            "portion_reduced_rated",
            "portion_standard_rated",
            "product_exempt",
            "product_exempt_holiday",
            "proportionally_rated",
            "reduced_rated",
            "reverse_charge",
            "standard_rated",
            "taxable_basis_reduced",
            "zero_rated",
        ]
        """
        The reasoning behind this tax, for example, if the product is tax exempt. The possible values for this field may be extended as new tax rules are supported.
        """
        taxable_amount: int
        """
        The amount on which tax is calculated, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
        """
        _inner_class_types = {
            "jurisdiction": Jurisdiction,
            "tax_rate_details": TaxRateDetails,
        }

    amount: int
    """
    The line item amount in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units). If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes were calculated on top of this amount.
    """
    amount_tax: int
    """
    The amount of tax calculated for this line item, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["tax.calculation_line_item"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    performance_location: Optional[str]
    """
    Indicates the line item represents a performance where the venue location might determine the tax, not the customer address. Leave empty if the tax code doesn't require a tax location. If you provide this value for tax codes with an `optional` location requirement, it overrides the customer address.
    """
    performance_location_details: Optional[PerformanceLocationDetails]
    """
    The address of the location where this line item's event or service takes place. Depending on the [tax code](https://docs.stripe.com/tax/tax-codes), providing a performance location is required, optional, or not supported. Use this to provide the address inline without pre-creating a [TaxLocation](https://docs.stripe.com/api/tax/location) object. Can't be used with `performance_location`.
    """
    product: Optional[str]
    """
    The ID of an existing [Product](https://docs.stripe.com/api/products/object).
    """
    quantity: int
    """
    The number of units of the item being purchased. For reversals, this is the quantity reversed.
    """
    reference: str
    """
    A custom identifier for this line item.
    """
    tax_behavior: Literal["exclusive", "inclusive"]
    """
    Specifies whether the `amount` includes taxes. If `tax_behavior=inclusive`, then the amount includes taxes.
    """
    tax_breakdown: Optional[List[TaxBreakdown]]
    """
    Detailed account of taxes relevant to this line item.
    """
    tax_code: str
    """
    The [tax code](https://docs.stripe.com/tax/tax-categories) ID used for this resource.
    """
    _inner_class_types = {
        "performance_location_details": PerformanceLocationDetails,
        "tax_breakdown": TaxBreakdown,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing._service_action_create_params import (
        ServiceActionCreateParams,
    )
    from stripe.params.v2.billing._service_action_retrieve_params import (
        ServiceActionRetrieveParams,
    )
    from stripe.params.v2.billing._service_action_update_params import (
        ServiceActionUpdateParams,
    )
    from stripe.v2.billing._service_action import ServiceAction


class ServiceActionService(StripeService):
    def create(
        self,
        params: "ServiceActionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ServiceAction":
        """
        Create a Service Action object.
        """
        return cast(
            "ServiceAction",
            self._request(
                "post",
                "/v2/billing/service_actions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ServiceActionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ServiceAction":
        """
        Create a Service Action object.
        """
        return cast(
            "ServiceAction",
            await self._request_async(
                "post",
                "/v2/billing/service_actions",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["ServiceActionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ServiceAction":
        """
        Retrieve a Service Action object.
        """
        return cast(
            "ServiceAction",
            self._request(
                "get",
                "/v2/billing/service_actions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ServiceActionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ServiceAction":
        """
        Retrieve a Service Action object.
        """
        return cast(
            "ServiceAction",
            await self._request_async(
                "get",
                "/v2/billing/service_actions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["ServiceActionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ServiceAction":
        """
        Update a ServiceAction object.
        """
        return cast(
            "ServiceAction",
            self._request(
                "post",
                "/v2/billing/service_actions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["ServiceActionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ServiceAction":
        """
        Update a ServiceAction object.
        """
        return cast(
            "ServiceAction",
            await self._request_async(
                "post",
                "/v2/billing/service_actions/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class DisputeCloseParams(TypedDict):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    status: Literal["lost", "won"]
    """
    Whether to close the dispute as `won` or `lost`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.payments.settlement_allocation_intents._split_cancel_params import (
        SplitCancelParams,
    )
    from stripe.params.v2.payments.settlement_allocation_intents._split_create_params import (
        SplitCreateParams,
    )
    from stripe.params.v2.payments.settlement_allocation_intents._split_list_params import (
        SplitListParams,
    )
    from stripe.params.v2.payments.settlement_allocation_intents._split_retrieve_params import (
        SplitRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.payments._settlement_allocation_intent_split import (
        SettlementAllocationIntentSplit,
    )


class SplitService(StripeService):
    def list(
        self,
        settlement_allocation_intent_id: str,
        params: Optional["SplitListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SettlementAllocationIntentSplit]":
        """
        List SettlementAllocationIntentSplits API.
        """
        return cast(
            "ListObject[SettlementAllocationIntentSplit]",
            self._request(
                "get",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        settlement_allocation_intent_id: str,
        params: Optional["SplitListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SettlementAllocationIntentSplit]":
        """
        List SettlementAllocationIntentSplits API.
        """
        return cast(
            "ListObject[SettlementAllocationIntentSplit]",
            await self._request_async(
                "get",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        settlement_allocation_intent_id: str,
        params: "SplitCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SettlementAllocationIntentSplit":
        """
        Create SettlementAllocationIntentSplit API.
        """
        return cast(
            "SettlementAllocationIntentSplit",
            self._request(
                "post",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        settlement_allocation_intent_id: str,
        params: "SplitCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "SettlementAllocationIntentSplit":
        """
        Create SettlementAllocationIntentSplit API.
        """
        return cast(
            "SettlementAllocationIntentSplit",
            await self._request_async(
                "post",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        settlement_allocation_intent_id: str,
        id: str,
        params: Optional["SplitRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SettlementAllocationIntentSplit":
        """
        Retrieve SettlementAllocationIntentSplit API.
        """
        return cast(
            "SettlementAllocationIntentSplit",
            self._request(
                "get",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits/{id}".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        settlement_allocation_intent_id: str,
        id: str,
        params: Optional["SplitRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SettlementAllocationIntentSplit":
        """
        Retrieve SettlementAllocationIntentSplit API.
        """
        return cast(
            "SettlementAllocationIntentSplit",
            await self._request_async(
                "get",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits/{id}".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        settlement_allocation_intent_id: str,
        id: str,
        params: Optional["SplitCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SettlementAllocationIntentSplit":
        """
        Cancel SettlementAllocationIntentSplit API.
        """
        return cast(
            "SettlementAllocationIntentSplit",
            self._request(
                "post",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits/{id}/cancel".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        settlement_allocation_intent_id: str,
        id: str,
        params: Optional["SplitCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "SettlementAllocationIntentSplit":
        """
        Cancel SettlementAllocationIntentSplit API.
        """
        return cast(
            "SettlementAllocationIntentSplit",
            await self._request_async(
                "post",
                "/v2/payments/settlement_allocation_intents/{settlement_allocation_intent_id}/splits/{id}/cancel".format(
                    settlement_allocation_intent_id=sanitize_id(
                        settlement_allocation_intent_id
                    ),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class FeeEntryListParams(TypedDict):
    collection_record: NotRequired[str]
    """
    Filter by money movement id (e.g. txn_xxx, bt_xxx).
    """
    created: NotRequired[str]
    """
    Filter for FeeEntries created at the exact specified timestamp.
    """
    created_gt: NotRequired[str]
    """
    Filter for FeeEntries created after the specified timestamp (exclusive).
    """
    created_gte: NotRequired[str]
    """
    Filter for FeeEntries created at or after the specified timestamp (inclusive).
    """
    created_lt: NotRequired[str]
    """
    Filter for FeeEntries created before the specified timestamp (exclusive).
    """
    created_lte: NotRequired[str]
    """
    Filter for FeeEntries created at or before the specified timestamp (inclusive).
    """
    fee_batch: NotRequired[str]
    """
    Filter by fee batch id (fb_xxx).
    """
    incurred_by: NotRequired[str]
    """
    Filter by usage object id (e.g. ch_xxx, py_xxx).
    """
    limit: NotRequired[int]
    """
    Maximum number of results to return per page. Defaults to 20.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired


class PayoutModifyParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._gift_card_operation import GiftCardOperation
    from stripe.params._gift_card_activate_params import GiftCardActivateParams
    from stripe.params._gift_card_cashout_params import GiftCardCashoutParams
    from stripe.params._gift_card_check_balance_params import (
        GiftCardCheckBalanceParams,
    )
    from stripe.params._gift_card_create_params import GiftCardCreateParams
    from stripe.params._gift_card_reload_params import GiftCardReloadParams
    from stripe.params._gift_card_retrieve_params import GiftCardRetrieveParams
    from stripe.params._gift_card_void_operation_params import (
        GiftCardVoidOperationParams,
    )


class GiftCard(CreateableAPIResource["GiftCard"]):
    """
    Represents third-party gift cards that can be used as a payment method through Stripe.
    """

    OBJECT_NAME: ClassVar[Literal["gift_card"]] = "gift_card"
    brand: Literal["fiserv_valuelink", "givex", "svs"]
    """
    The brand of the gift card.
    """
    exp_month: Optional[int]
    """
    The expiration month of the gift card.
    """
    exp_year: Optional[int]
    """
    The expiration year of the gift card.
    """
    fingerprint: str
    """
    Uniquely identifies this particular gift card number. You can use this attribute to check whether two gift cards are the same.
    """
    id: str
    """
    Unique identifier for the object.
    """
    last4: Optional[str]
    """
    The last four digits of the gift card number.
    """
    last_operation: Optional[ExpandableField["GiftCardOperation"]]
    """
    The last operation performed on this gift card.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["gift_card"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """

    @classmethod
    def _cls_activate(
        cls, gift_card: str, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        return cast(
            "GiftCardOperation",
            cls._static_request(
                "post",
                "/v1/gift_cards/{gift_card}/activate".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def activate(
        gift_card: str, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        ...

    @overload
    def activate(
        self, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        ...

    @class_method_variant("_cls_activate")
    def activate(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        return cast(
            "GiftCardOperation",
            self._request(
                "post",
                "/v1/gift_cards/{gift_card}/activate".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_activate_async(
        cls, gift_card: str, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        return cast(
            "GiftCardOperation",
            await cls._static_request_async(
                "post",
                "/v1/gift_cards/{gift_card}/activate".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def activate_async(
        gift_card: str, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        ...

    @overload
    async def activate_async(
        self, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        ...

    @class_method_variant("_cls_activate_async")
    async def activate_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardActivateParams"]
    ) -> "GiftCardOperation":
        """
        Activates a third-party gift card and optionally sets its balance.
        """
        return cast(
            "GiftCardOperation",
            await self._request_async(
                "post",
                "/v1/gift_cards/{gift_card}/activate".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def _cls_cashout(
        cls, gift_card: str, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        return cast(
            "GiftCardOperation",
            cls._static_request(
                "post",
                "/v1/gift_cards/{gift_card}/cashout".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def cashout(
        gift_card: str, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        ...

    @overload
    def cashout(
        self, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        ...

    @class_method_variant("_cls_cashout")
    def cashout(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        return cast(
            "GiftCardOperation",
            self._request(
                "post",
                "/v1/gift_cards/{gift_card}/cashout".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_cashout_async(
        cls, gift_card: str, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        return cast(
            "GiftCardOperation",
            await cls._static_request_async(
                "post",
                "/v1/gift_cards/{gift_card}/cashout".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def cashout_async(
        gift_card: str, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        ...

    @overload
    async def cashout_async(
        self, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        ...

    @class_method_variant("_cls_cashout_async")
    async def cashout_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardCashoutParams"]
    ) -> "GiftCardOperation":
        """
        Cashout a third-party gift card by zeroing its balance.
        """
        return cast(
            "GiftCardOperation",
            await self._request_async(
                "post",
                "/v1/gift_cards/{gift_card}/cashout".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def _cls_check_balance(
        cls, gift_card: str, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        return cast(
            "GiftCardOperation",
            cls._static_request(
                "post",
                "/v1/gift_cards/{gift_card}/check_balance".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def check_balance(
        gift_card: str, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        ...

    @overload
    def check_balance(
        self, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        ...

    @class_method_variant("_cls_check_balance")
    def check_balance(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        return cast(
            "GiftCardOperation",
            self._request(
                "post",
                "/v1/gift_cards/{gift_card}/check_balance".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_check_balance_async(
        cls, gift_card: str, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        return cast(
            "GiftCardOperation",
            await cls._static_request_async(
                "post",
                "/v1/gift_cards/{gift_card}/check_balance".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def check_balance_async(
        gift_card: str, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        ...

    @overload
    async def check_balance_async(
        self, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        ...

    @class_method_variant("_cls_check_balance_async")
    async def check_balance_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardCheckBalanceParams"]
    ) -> "GiftCardOperation":
        """
        Checks the balance of a third-party gift card.
        """
        return cast(
            "GiftCardOperation",
            await self._request_async(
                "post",
                "/v1/gift_cards/{gift_card}/check_balance".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def create(cls, **params: Unpack["GiftCardCreateParams"]) -> "GiftCard":
        """
        Creates a gift card object.
        """
        return cast(
            "GiftCard",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["GiftCardCreateParams"]
    ) -> "GiftCard":
        """
        Creates a gift card object.
        """
        return cast(
            "GiftCard",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_reload(
        cls, gift_card: str, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        return cast(
            "GiftCardOperation",
            cls._static_request(
                "post",
                "/v1/gift_cards/{gift_card}/reload".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def reload(
        gift_card: str, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        ...

    @overload
    def reload(
        self, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        ...

    @class_method_variant("_cls_reload")
    def reload(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        return cast(
            "GiftCardOperation",
            self._request(
                "post",
                "/v1/gift_cards/{gift_card}/reload".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_reload_async(
        cls, gift_card: str, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        return cast(
            "GiftCardOperation",
            await cls._static_request_async(
                "post",
                "/v1/gift_cards/{gift_card}/reload".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def reload_async(
        gift_card: str, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        ...

    @overload
    async def reload_async(
        self, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        ...

    @class_method_variant("_cls_reload_async")
    async def reload_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardReloadParams"]
    ) -> "GiftCardOperation":
        """
        Reloads a third-party gift card by adding the specified amount to its balance.
        """
        return cast(
            "GiftCardOperation",
            await self._request_async(
                "post",
                "/v1/gift_cards/{gift_card}/reload".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["GiftCardRetrieveParams"]
    ) -> "GiftCard":
        """
        Retrieves a third-party gift card object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["GiftCardRetrieveParams"]
    ) -> "GiftCard":
        """
        Retrieves a third-party gift card object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_void_operation(
        cls, gift_card: str, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        return cast(
            "GiftCardOperation",
            cls._static_request(
                "post",
                "/v1/gift_cards/{gift_card}/void_operation".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def void_operation(
        gift_card: str, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        ...

    @overload
    def void_operation(
        self, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        ...

    @class_method_variant("_cls_void_operation")
    def void_operation(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        return cast(
            "GiftCardOperation",
            self._request(
                "post",
                "/v1/gift_cards/{gift_card}/void_operation".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_void_operation_async(
        cls, gift_card: str, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        return cast(
            "GiftCardOperation",
            await cls._static_request_async(
                "post",
                "/v1/gift_cards/{gift_card}/void_operation".format(
                    gift_card=sanitize_id(gift_card)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def void_operation_async(
        gift_card: str, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        ...

    @overload
    async def void_operation_async(
        self, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        ...

    @class_method_variant("_cls_void_operation_async")
    async def void_operation_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["GiftCardVoidOperationParams"]
    ) -> "GiftCardOperation":
        """
        Voids a previously performed gift card operation.
        """
        return cast(
            "GiftCardOperation",
            await self._request_async(
                "post",
                "/v1/gift_cards/{gift_card}/void_operation".format(
                    gift_card=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class GbBankAccountInitiateConfirmationOfPayeeParams(TypedDict):
    business_type: NotRequired[Literal["business", "personal"]]
    """
    The business type to be checked against. Legal entity information will be used if unspecified.
    """
    name: NotRequired[str]
    """
    The name of the user to be checked against. Legal entity information will be used if unspecified.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.core.accounts._person_create_params import (
        PersonCreateParams,
    )
    from stripe.params.v2.core.accounts._person_delete_params import (
        PersonDeleteParams,
    )
    from stripe.params.v2.core.accounts._person_list_params import (
        PersonListParams,
    )
    from stripe.params.v2.core.accounts._person_retrieve_params import (
        PersonRetrieveParams,
    )
    from stripe.params.v2.core.accounts._person_update_params import (
        PersonUpdateParams,
    )
    from stripe.v2._deleted_object import DeletedObject
    from stripe.v2._list_object import ListObject
    from stripe.v2.core._account_person import AccountPerson


class PersonService(StripeService):
    def list(
        self,
        account_id: str,
        params: Optional["PersonListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[AccountPerson]":
        """
        Returns a paginated list of Persons associated with an Account.
        """
        return cast(
            "ListObject[AccountPerson]",
            self._request(
                "get",
                "/v2/core/accounts/{account_id}/persons".format(
                    account_id=sanitize_id(account_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        account_id: str,
        params: Optional["PersonListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[AccountPerson]":
        """
        Returns a paginated list of Persons associated with an Account.
        """
        return cast(
            "ListObject[AccountPerson]",
            await self._request_async(
                "get",
                "/v2/core/accounts/{account_id}/persons".format(
                    account_id=sanitize_id(account_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        account_id: str,
        params: Optional["PersonCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountPerson":
        """
        Create a Person. Adds an individual to an Account's identity. You can set relationship attributes and identity information at creation.
        """
        return cast(
            "AccountPerson",
            self._request(
                "post",
                "/v2/core/accounts/{account_id}/persons".format(
                    account_id=sanitize_id(account_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        account_id: str,
        params: Optional["PersonCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountPerson":
        """
        Create a Person. Adds an individual to an Account's identity. You can set relationship attributes and identity information at creation.
        """
        return cast(
            "AccountPerson",
            await self._request_async(
                "post",
                "/v2/core/accounts/{account_id}/persons".format(
                    account_id=sanitize_id(account_id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def delete(
        self,
        account_id: str,
        id: str,
        params: Optional["PersonDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "DeletedObject":
        """
        Delete a Person associated with an Account.
        """
        return cast(
            "DeletedObject",
            self._request(
                "delete",
                "/v2/core/accounts/{account_id}/persons/{id}".format(
                    account_id=sanitize_id(account_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        account_id: str,
        id: str,
        params: Optional["PersonDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "DeletedObject":
        """
        Delete a Person associated with an Account.
        """
        return cast(
            "DeletedObject",
            await self._request_async(
                "delete",
                "/v2/core/accounts/{account_id}/persons/{id}".format(
                    account_id=sanitize_id(account_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        account_id: str,
        id: str,
        params: Optional["PersonRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountPerson":
        """
        Retrieves a Person associated with an Account.
        """
        return cast(
            "AccountPerson",
            self._request(
                "get",
                "/v2/core/accounts/{account_id}/persons/{id}".format(
                    account_id=sanitize_id(account_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        account_id: str,
        id: str,
        params: Optional["PersonRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountPerson":
        """
        Retrieves a Person associated with an Account.
        """
        return cast(
            "AccountPerson",
            await self._request_async(
                "get",
                "/v2/core/accounts/{account_id}/persons/{id}".format(
                    account_id=sanitize_id(account_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        account_id: str,
        id: str,
        params: Optional["PersonUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountPerson":
        """
        Updates a Person associated with an Account.
        """
        return cast(
            "AccountPerson",
            self._request(
                "post",
                "/v2/core/accounts/{account_id}/persons/{id}".format(
                    account_id=sanitize_id(account_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        account_id: str,
        id: str,
        params: Optional["PersonUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "AccountPerson":
        """
        Updates a Person associated with an Account.
        """
        return cast(
            "AccountPerson",
            await self._request_async(
                "post",
                "/v2/core/accounts/{account_id}/persons/{id}".format(
                    account_id=sanitize_id(account_id),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class ShippingRateCreateParams(RequestOptions):
    delivery_estimate: NotRequired["ShippingRateCreateParamsDeliveryEstimate"]
    """
    The estimated range for how long shipping will take, meant to be displayable to the customer. This will appear on CheckoutSessions.
    """
    display_name: str
    """
    The name of the shipping rate, meant to be displayable to the customer. This will appear on CheckoutSessions.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    fixed_amount: NotRequired["ShippingRateCreateParamsFixedAmount"]
    """
    Describes a fixed amount to charge for shipping. Must be present if type is `fixed_amount`.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Specifies whether the rate is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`.
    """
    tax_code: NotRequired[str]
    """
    A [tax code](https://docs.stripe.com/tax/tax-categories) ID. The Shipping tax code is `txcd_92010001`.
    """
    type: NotRequired[Literal["fixed_amount"]]
    """
    The type of calculation to use on the shipping rate.
    """


class ShippingRateCreateParamsDeliveryEstimate(TypedDict):
    maximum: NotRequired["ShippingRateCreateParamsDeliveryEstimateMaximum"]
    """
    The upper bound of the estimated range. If empty, represents no upper bound i.e., infinite.
    """
    minimum: NotRequired["ShippingRateCreateParamsDeliveryEstimateMinimum"]
    """
    The lower bound of the estimated range. If empty, represents no lower bound.
    """


class ShippingRateCreateParamsDeliveryEstimateMaximum(TypedDict):
    unit: Literal["business_day", "day", "hour", "month", "week"]
    """
    A unit of time.
    """
    value: int
    """
    Must be greater than 0.
    """


class ShippingRateCreateParamsDeliveryEstimateMinimum(TypedDict):
    unit: Literal["business_day", "day", "hour", "month", "week"]
    """
    A unit of time.
    """
    value: int
    """
    Must be greater than 0.
    """


class ShippingRateCreateParamsFixedAmount(TypedDict):
    amount: int
    """
    A non-negative integer in cents representing how much to charge.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    currency_options: NotRequired[
        "Dict[str, ShippingRateCreateParamsFixedAmountCurrencyOptions]|UntypedStripeObject[ShippingRateCreateParamsFixedAmountCurrencyOptions]"
    ]
    """
    Shipping rates defined in each available currency option. Each key must be a three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html) and a [supported currency](https://stripe.com/docs/currencies).
    """


class ShippingRateCreateParamsFixedAmountCurrencyOptions(TypedDict):
    amount: int
    """
    A non-negative integer in cents representing how much to charge.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Specifies whether the rate is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._plan import Plan
    from stripe._request_options import RequestOptions
    from stripe.params._plan_create_params import PlanCreateParams
    from stripe.params._plan_delete_params import PlanDeleteParams
    from stripe.params._plan_list_params import PlanListParams
    from stripe.params._plan_retrieve_params import PlanRetrieveParams
    from stripe.params._plan_update_params import PlanUpdateParams


class PlanService(StripeService):
    def delete(
        self,
        plan: str,
        params: Optional["PlanDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        Deleting plans means new subscribers can't be added. Existing subscribers aren't affected.
        """
        return cast(
            "Plan",
            self._request(
                "delete",
                "/v1/plans/{plan}".format(plan=sanitize_id(plan)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        plan: str,
        params: Optional["PlanDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        Deleting plans means new subscribers can't be added. Existing subscribers aren't affected.
        """
        return cast(
            "Plan",
            await self._request_async(
                "delete",
                "/v1/plans/{plan}".format(plan=sanitize_id(plan)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        plan: str,
        params: Optional["PlanRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        Retrieves the plan with the given ID.
        """
        return cast(
            "Plan",
            self._request(
                "get",
                "/v1/plans/{plan}".format(plan=sanitize_id(plan)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        plan: str,
        params: Optional["PlanRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        Retrieves the plan with the given ID.
        """
        return cast(
            "Plan",
            await self._request_async(
                "get",
                "/v1/plans/{plan}".format(plan=sanitize_id(plan)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        plan: str,
        params: Optional["PlanUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        Updates the specified plan by setting the values of the parameters passed. Any parameters not provided are left unchanged. By design, you cannot change a plan's ID, amount, currency, or billing cycle.
        """
        return cast(
            "Plan",
            self._request(
                "post",
                "/v1/plans/{plan}".format(plan=sanitize_id(plan)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        plan: str,
        params: Optional["PlanUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        Updates the specified plan by setting the values of the parameters passed. Any parameters not provided are left unchanged. By design, you cannot change a plan's ID, amount, currency, or billing cycle.
        """
        return cast(
            "Plan",
            await self._request_async(
                "post",
                "/v1/plans/{plan}".format(plan=sanitize_id(plan)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: Optional["PlanListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Plan]":
        """
        Returns a list of your plans.
        """
        return cast(
            "ListObject[Plan]",
            self._request(
                "get",
                "/v1/plans",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["PlanListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Plan]":
        """
        Returns a list of your plans.
        """
        return cast(
            "ListObject[Plan]",
            await self._request_async(
                "get",
                "/v1/plans",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "PlanCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        You can now model subscriptions more flexibly using the [Prices API](https://docs.stripe.com/api#prices). It replaces the Plans API and is backwards compatible to simplify your migration.
        """
        return cast(
            "Plan",
            self._request(
                "post",
                "/v1/plans",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "PlanCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Plan":
        """
        You can now model subscriptions more flexibly using the [Prices API](https://docs.stripe.com/api#prices). It replaces the Plans API and is backwards compatible to simplify your migration.
        """
        return cast(
            "Plan",
            await self._request_async(
                "post",
                "/v1/plans",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class BillSettingVersion(StripeObject):
    """
    A Bill Setting Version is a specific configuration of a BillSetting at a point in time. Bill Setting Versions enable you to track changes to bill generation and invoice settings over time and manage which version is active for new billing operations.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.bill_setting_version"]] = (
        "v2.billing.bill_setting_version"
    )

    class Calculation(StripeObject):
        class Tax(StripeObject):
            type: Literal["automatic", "manual"]
            """
            Determines if tax is calculated automatically based on a PTC or manually based on rules defined by the business. Defaults to "manual".
            """

        tax: Optional[Tax]
        """
        Settings for calculating tax.
        """
        _inner_class_types = {"tax": Tax}

    class Invoice(StripeObject):
        class TimeUntilDue(StripeObject):
            interval: Literal["day", "month", "week", "year"]
            """
            The interval unit for the time until due.
            """
            interval_count: int
            """
            The number of interval units. For example, if interval=day and interval_count=30,
            the invoice is due in 30 days.
            """

        time_until_due: Optional[TimeUntilDue]
        """
        The amount of time until the invoice is overdue for payment.
        """
        _inner_class_types = {"time_until_due": TimeUntilDue}

    calculation: Optional[Calculation]
    """
    Settings related to calculating a bill.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    id: str
    """
    The ID of the BillSettingVersion object.
    """
    invoice: Optional[Invoice]
    """
    Settings related to invoice behavior.
    """
    invoice_rendering_template: Optional[str]
    """
    The ID of the invoice rendering template to be used when generating invoices.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.billing.bill_setting_version"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    _inner_class_types = {"calculation": Calculation, "invoice": Invoice}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class AccountLinkCreateParams(RequestOptions):
    account: str
    """
    The identifier of the account to create an account link for.
    """
    collect: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    The collect parameter is deprecated. Use `collection_options` instead.
    """
    collection_options: NotRequired["AccountLinkCreateParamsCollectionOptions"]
    """
    Specifies the requirements that Stripe collects from connected accounts in the Connect Onboarding flow.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    refresh_url: NotRequired[str]
    """
    The URL the user will be redirected to if the account link is expired, has been previously-visited, or is otherwise invalid. The URL you specify should attempt to generate a new account link with the same parameters used to create the original account link, then redirect the user to the new account link's URL so they can continue with Connect Onboarding. If a new account link cannot be generated or the redirect fails you should display a useful error to the user.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon leaving or completing the linked flow.
    """
    type: Literal[
        "account_onboarding",
        "account_update",
        "capital_financing_offer",
        "capital_financing_reporting",
    ]
    """
    The type of account link the user is requesting.

    You can create Account Links of type `account_update` only for connected accounts where your platform is responsible for collecting requirements, including Custom accounts. You can't create them for accounts that have access to a Stripe-hosted Dashboard. If you use [Connect embedded components](https://docs.stripe.com/connect/get-started-connect-embedded-components), you can include components that allow your connected accounts to update their own information. For an account without Stripe-hosted Dashboard access where Stripe is liable for negative balances, you must use embedded components.
    """


class AccountLinkCreateParamsCollectionOptions(TypedDict):
    external_account_collection: NotRequired[bool]
    """
    Specifies whether the platform collects external account information from connected accounts during Connect Onboarding. When set to `true`, the platform only collects external account information if the connected account has capabilities which require it. Some capabilities, such as `treasury`, don't require external account collection. When set to `false`, external account collection is skipped. Defaults to `true`.
    """
    fields: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). If you don't specify `collection_options`, the default value is `currently_due`.
    """
    future_requirements: NotRequired[Literal["include", "omit"]]
    """
    Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class RateCardSubscription(StripeObject):
    """
    A Rate Card Subscription represents a customer's subscription to a Rate Card's usage-based pricing. It tracks the
    servicing status (whether usage is being measured) and collection status (whether payments are current). Subscriptions
    bill according to the service interval defined in the Rate Card and are collected on the associated Billing Cadence.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.rate_card_subscription"]] = (
        "v2.billing.rate_card_subscription"
    )

    class CollectionStatusTransitions(StripeObject):
        awaiting_customer_action_at: Optional[str]
        """
        When the collection status transitioned to awaiting customer action.
        """
        current_at: Optional[str]
        """
        When the collection status transitioned to current.
        """
        past_due_at: Optional[str]
        """
        When the collection status transitioned to past due.
        """
        paused_at: Optional[str]
        """
        When the collection status transitioned to paused.
        """
        unpaid_at: Optional[str]
        """
        When the collection status transitioned to unpaid.
        """

    class ServicingStatusTransitions(StripeObject):
        activated_at: Optional[str]
        """
        When the servicing status transitioned to activated.
        """
        canceled_at: Optional[str]
        """
        When the servicing status transitioned to canceled.
        """
        paused_at: Optional[str]
        """
        When the servicing status transitioned to paused.
        """
        will_activate_at: Optional[str]
        """
        When the servicing is scheduled to transition to activate.
        """
        will_cancel_at: Optional[str]
        """
        When the servicing is scheduled to cancel.
        """

    billing_cadence: str
    """
    The ID of the Billing Cadence.
    """
    collection_status: Optional[
        Literal[
            "awaiting_customer_action",
            "current",
            "past_due",
            "paused",
            "unpaid",
        ]
    ]
    """
    The payment status of a Rate Card Subscription.
    """
    collection_status_transitions: Optional[CollectionStatusTransitions]
    """
    The collection status transitions of the Rate Card Subscription.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.billing.rate_card_subscription"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    rate_card: str
    """
    The ID of the Rate Card.
    """
    rate_card_version: str
    """
    The ID of the Rate Card Version.
    """
    servicing_status: Optional[
        Literal["active", "canceled", "paused", "pending"]
    ]
    """
    The servicing status of a Rate Card Subscription.
    """
    servicing_status_transitions: Optional[ServicingStatusTransitions]
    """
    The servicing status transitions of the Rate Card Subscription.
    """
    test_clock: Optional[str]
    """
    The ID of the Test Clock, if any.
    """
    _inner_class_types = {
        "collection_status_transitions": CollectionStatusTransitions,
        "servicing_status_transitions": ServicingStatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class ConnectionSessionCreateParams(TypedDict):
    account: str
    """
    The Account the ConnectionSession will create a connection for.
    """
    allowed_connection_types: NotRequired[List[Literal["link"]]]
    """
    The Connection types that the ConnectionSession is allowed to establish.
    """
    requested_access: NotRequired[List[Literal["payout_methods"]]]
    """
    The access that should be collected with the ConnectionSession.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._confirmation_token import ConfirmationToken
    from stripe._request_options import RequestOptions
    from stripe.params._confirmation_token_retrieve_params import (
        ConfirmationTokenRetrieveParams,
    )


class ConfirmationTokenService(StripeService):
    def retrieve(
        self,
        confirmation_token: str,
        params: Optional["ConfirmationTokenRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ConfirmationToken":
        """
        Retrieves an existing ConfirmationToken object
        """
        return cast(
            "ConfirmationToken",
            self._request(
                "get",
                "/v1/confirmation_tokens/{confirmation_token}".format(
                    confirmation_token=sanitize_id(confirmation_token),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        confirmation_token: str,
        params: Optional["ConfirmationTokenRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ConfirmationToken":
        """
        Retrieves an existing ConfirmationToken object
        """
        return cast(
            "ConfirmationToken",
            await self._request_async(
                "get",
                "/v1/confirmation_tokens/{confirmation_token}".format(
                    confirmation_token=sanitize_id(confirmation_token),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
import json
from stripe._api_version import _ApiVersion
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from uuid import uuid4
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._customer_balance_transaction import CustomerBalanceTransaction
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._customer_balance_transaction_create_params import (
        CustomerBalanceTransactionCreateParams,
    )
    from stripe.params._customer_balance_transaction_list_params import (
        CustomerBalanceTransactionListParams,
    )
    from stripe.params._customer_balance_transaction_retrieve_params import (
        CustomerBalanceTransactionRetrieveParams,
    )
    from stripe.params._customer_balance_transaction_update_params import (
        CustomerBalanceTransactionUpdateParams,
    )


class CustomerBalanceTransactionService(StripeService):
    def list(
        self,
        customer: str,
        params: Optional["CustomerBalanceTransactionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CustomerBalanceTransaction]":
        """
        Returns a list of transactions that updated the customer's [balances](https://docs.stripe.com/docs/billing/customer/balance).
        """
        return cast(
            "ListObject[CustomerBalanceTransaction]",
            self._request(
                "get",
                "/v1/customers/{customer}/balance_transactions".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        customer: str,
        params: Optional["CustomerBalanceTransactionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[CustomerBalanceTransaction]":
        """
        Returns a list of transactions that updated the customer's [balances](https://docs.stripe.com/docs/billing/customer/balance).
        """
        return cast(
            "ListObject[CustomerBalanceTransaction]",
            await self._request_async(
                "get",
                "/v1/customers/{customer}/balance_transactions".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        customer: str,
        params: "CustomerBalanceTransactionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerBalanceTransaction":
        """
        Creates an immutable transaction that updates the customer's credit [balance](https://docs.stripe.com/docs/billing/customer/balance).
        """
        return cast(
            "CustomerBalanceTransaction",
            self._request(
                "post",
                "/v1/customers/{customer}/balance_transactions".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        customer: str,
        params: "CustomerBalanceTransactionCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerBalanceTransaction":
        """
        Creates an immutable transaction that updates the customer's credit [balance](https://docs.stripe.com/docs/billing/customer/balance).
        """
        return cast(
            "CustomerBalanceTransaction",
            await self._request_async(
                "post",
                "/v1/customers/{customer}/balance_transactions".format(
                    customer=sanitize_id(customer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        customer: str,
        transaction: str,
        params: Optional["CustomerBalanceTransactionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerBalanceTransaction":
        """
        Retrieves a specific customer balance transaction that updated the customer's [balances](https://docs.stripe.com/docs/billing/customer/balance).
        """
        return cast(
            "CustomerBalanceTransaction",
            self._request(
                "get",
                "/v1/customers/{customer}/balance_transactions/{transaction}".format(
                    customer=sanitize_id(customer),
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        customer: str,
        transaction: str,
        params: Optional["CustomerBalanceTransactionRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerBalanceTransaction":
        """
        Retrieves a specific customer balance transaction that updated the customer's [balances](https://docs.stripe.com/docs/billing/customer/balance).
        """
        return cast(
            "CustomerBalanceTransaction",
            await self._request_async(
                "get",
                "/v1/customers/{customer}/balance_transactions/{transaction}".format(
                    customer=sanitize_id(customer),
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        customer: str,
        transaction: str,
        params: Optional["CustomerBalanceTransactionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerBalanceTransaction":
        """
        Most credit balance transaction fields are immutable, but you may update its description and metadata.
        """
        return cast(
            "CustomerBalanceTransaction",
            self._request(
                "post",
                "/v1/customers/{customer}/balance_transactions/{transaction}".format(
                    customer=sanitize_id(customer),
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        customer: str,
        transaction: str,
        params: Optional["CustomerBalanceTransactionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "CustomerBalanceTransaction":
        """
        Most credit balance transaction fields are immutable, but you may update its description and metadata.
        """
        return cast(
            "CustomerBalanceTransaction",
            await self._request_async(
                "post",
                "/v1/customers/{customer}/balance_transactions/{transaction}".format(
                    customer=sanitize_id(customer),
                    transaction=sanitize_id(transaction),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def serialize_batch_create(
        self,
        customer: str,
        params: Optional["CustomerBalanceTransactionCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a CustomerBalanceTransaction create request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"customer": customer},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)

    def serialize_batch_update(
        self,
        customer: str,
        transaction: str,
        params: Optional["CustomerBalanceTransactionUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> str:
        """
        Serializes a CustomerBalanceTransaction update request into a batch job JSONL line.
        """
        item_id = str(uuid4())
        stripe_version = (
            options.get("stripe_version") if options else None
        ) or _ApiVersion.CURRENT
        context = options.get("stripe_context") if options else None
        batch_request = {
            "id": item_id,
            "path_params": {"customer": customer, "transaction": transaction},
            "params": params,
            "stripe_version": stripe_version,
        }
        if context is not None:
            batch_request["context"] = context
        return json.dumps(batch_request)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import NotRequired, TypedDict


class CustomPricingUnitListParams(TypedDict):
    active: NotRequired[bool]
    """
    Filter for active/inactive custom pricing units. Mutually exclusive with `lookup_keys`.
    """
    limit: NotRequired[int]
    """
    Optionally set the maximum number of results per page. Defaults to 20.
    """
    lookup_keys: NotRequired[List[str]]
    """
    Filter by lookup keys. Mutually exclusive with `active`.
    You can specify up to 10 lookup keys.
    """



import datetime
from decimal import Decimal
import json
import pickle
from copy import copy, deepcopy

import pytest

import stripe
from stripe._invoice import Invoice
from stripe._stripe_object import StripeObject

# We use this because it has a map, "restriction.currency_options" from string -> CurrencyOptions nested class.
SAMPLE_PROMOTION_CODE = json.loads(
    """{
  "id": "promo_1NzO4FIFdHa3DensTcbAA0mz",
  "object": "promotion_code",
  "restrictions": {
    "currency_options": {
      "gbp": {
        "minimum_amount": 10
      },
      "usd": {
        "minimum_amount": 5
      }
    }
  }
}
"""
)

SAMPLE_INVOICE = json.loads(
    """{
  "id": "in_xyz",
  "object": "invoice",
  "automatic_tax": {
    "enabled": false,
    "status": null
  },
  "created": 1694725031,
  "lines": {
    "object": "list",
    "data": [
      {
        "id": "sli_xyz",
        "object": "line_item",
        "period": {
          "end": 1697316491,
          "start": 1694724491
        },
        "plan": {
          "id": "price_xyz",
          "object": "plan"
        },
        "price": {
          "id": "price_xyz",
          "object": "price",
          "billing_scheme": "per_unit",
          "recurring": {
            "aggregate_usage": null,
            "interval": "month",
            "interval_count": 1,
            "trial_period_days": null,
            "usage_type": "licensed"
          }
        }
      }
    ],
    "has_more": false,
    "total_count": 1,
    "url": "/v1/invoices/in_1NqMb5IFdHa3DenshEllwgEX/lines"
  },
  "livemode": false,
  "next_payment_attempt": null,
  "shipping_cost": {
    "taxes": [{"amount": 10, "rate": {"id": "rate_xyz", "object": "tax_rate", "active": false}}]
  }
}
"""
)


class TestStripeObject(object):
    def test_initializes_with_parameters(self):
        obj = StripeObject("foo", "bar", myparam=5, yourparam="boo")

        assert obj.id == "foo"
        assert obj.api_key == "bar"

    def test_access(self):
        obj = StripeObject("myid", "mykey", myparam=5)

        # Empty
        with pytest.raises(AttributeError):
            obj.myattr
        with pytest.raises(KeyError):
            obj["myattr"]
        assert getattr(obj, "myattr", "def") == "def"
        assert "myattr" not in obj

        # Setters
        obj.myattr = "myval"
        obj["myitem"] = "itval"
        obj.mydef = getattr(obj, "mydef", "sdef")
        assert obj.mydef == "sdef"

        # Getters
        assert obj.myattr == "myval"
        assert obj["myattr"] == "myval"

        assert sorted(obj.to_dict().keys()) == [
            "id",
            "myattr",
            "mydef",
            "myitem",
        ]

        assert sorted(obj.to_dict().values()) == [
            "itval",
            "myid",
            "myval",
            "sdef",
        ]

        # Illegal operations
        with pytest.raises(ValueError):
            obj.foo = ""

    def test_refresh_from(self, mocker):
        obj = StripeObject.construct_from(
            {"foo": "bar", "trans": "me"}, "mykey"
        )

        assert obj.api_key == "mykey"
        assert obj.foo == "bar"
        assert obj["trans"] == "me"
        assert obj.stripe_version is stripe.api_version
        assert obj.stripe_account is None
        assert obj.last_response is None

        last_response = mocker.Mock()
        obj.refresh_from(
            {"foo": "baz", "johnny": 5},
            "key2",
            stripe_version="2017-08-15",
            stripe_account="acct_foo",
            last_response=last_response,
        )

        assert obj.johnny == 5
        assert obj.foo == "baz"
        with pytest.raises(AttributeError):
            obj.trans
        assert obj.api_key == "key2"
        assert obj.stripe_version == "2017-08-15"
        assert obj.stripe_account == "acct_foo"
        assert obj.last_response == last_response

        obj.refresh_from(
            {"trans": 4, "metadata": {"amount": 42}}, "key2", True
        )

        assert obj.foo == "baz"
        assert obj.trans == 4

    def test_passing_nested_refresh(self):
        obj = StripeObject.construct_from(
            {"foos": {"type": "list", "data": [{"id": "nested"}]}},
            "key",
            stripe_account="acct_foo",
        )

        nested = obj.foos.data[0]

        assert obj.api_key == "key"
        assert nested.id == "nested"
        assert nested.api_key == "key"
        assert nested.stripe_account == "acct_foo"

    def test_refresh_from_nested_object(self):
        obj = stripe.Invoice.construct_from(SAMPLE_INVOICE, "key")

        assert len(obj.lines) == 1
        # Does a list get deserialized to the correct class?
        assert isinstance(obj.lines, stripe.ListObject)
        # Do items within a list get deserialized to the correct class?
        assert isinstance(obj.lines.data[0], stripe.InvoiceLineItem)
        # Do references to other top-level resources get deserialized to the correct class?
        assert isinstance(obj.lines.data[0].price, stripe.Price)
        assert obj.lines.data[0].price.billing_scheme == "per_unit"
        # Do inner classes on referenced top-level resources get deserialized to the correct inner class.
        assert isinstance(
            obj.lines.data[0].price.recurring, stripe.Price.Recurring
        )
        assert isinstance(
            obj.lines.data[0].price.recurring, stripe.Price.Recurring
        )

        # Test: do nested objects get deserialized to the correct inner class?
        assert isinstance(obj.automatic_tax, stripe.Invoice.AutomaticTax)
        assert isinstance(obj.shipping_cost, stripe.Invoice.ShippingCost)
        # Test: do nested objects inside lists get deserialized to the correct inner class?
        assert isinstance(
            obj.shipping_cost.taxes[0], stripe.Invoice.ShippingCost.Tax
        )

    def test_refresh_from_nested_object_can_be_paged(self):
        obj = stripe.Invoice.construct_from(SAMPLE_INVOICE, "key")

        assert len(obj.lines) == 1
        assert isinstance(obj.lines, stripe.ListObject)
        seen = [item["id"] for item in obj.lines.auto_paging_iter()]

        assert seen == ["sli_xyz"]

        assert isinstance(obj.lines.data[0], stripe.InvoiceLineItem)
        assert isinstance(obj.lines.data[0].price, StripeObject)
        assert isinstance(obj.lines.data[0].price, stripe.Price)
        assert obj.lines.data[0].price.billing_scheme == "per_unit"

    def test_refresh_on_map_to_nested_object(self):
        obj = stripe.PromotionCode.construct_from(SAMPLE_PROMOTION_CODE, "key")

        assert isinstance(
            obj.restrictions.currency_options,
            dict,
        )
        assert not isinstance(
            obj.restrictions.currency_options,
            StripeObject,
        )
        assert isinstance(
            obj.restrictions.currency_options["gbp"],
            stripe.PromotionCode.Restrictions.CurrencyOptions,
        )

    def test_to_json(self):
        obj = StripeObject.construct_from(SAMPLE_INVOICE, "key")

        self.check_invoice_data(json.loads(str(obj)))

    def check_invoice_data(self, data):
        # Check rough structure
        assert len(list(data.keys())) == 8
        assert len(list(data["lines"]["data"][0].keys())) == 5
        assert len(data["lines"]["data"]) == 1

        # Check various data types
        assert data["created"] == 1694725031
        assert data["next_payment_attempt"] is None
        assert data["livemode"] is False
        assert (
            data["lines"]["data"][0]["price"]["billing_scheme"] == "per_unit"
        )

    def test_repr(self):
        obj = StripeObject("foo", "bar", myparam=5)

        obj["object"] = "\u4e00boo\u1f00"
        obj.date = datetime.datetime.fromtimestamp(1511136000)
        obj.dec = Decimal("1.23")

        res = repr(obj)

        assert "<StripeObject \u4e00boo\u1f00" in res
        assert "id=foo" in res
        assert '"date": 1511136000' in res
        assert '"dec": "1.23"' in res

    def test_pickling(self):
        obj = StripeObject("foo", "bar", myparam=5)

        obj["object"] = "boo"
        obj.refresh_from(
            {
                "fala": "lalala",
                # ensures that empty strings are correctly unpickled in Py3
                "emptystring": "",
            },
            api_key="bar",
            partial=True,
        )

        assert obj.fala == "lalala"

        pickled = pickle.dumps(obj)
        newobj = pickle.loads(pickled)

        assert newobj.id == "foo"
        assert newobj.api_key == "bar"
        assert newobj["object"] == "boo"
        assert newobj.fala == "lalala"
        assert newobj.emptystring == ""

    def test_deletion(self):
        obj = StripeObject("id", "key")

        obj.coupon = "foo"
        assert obj.coupon == "foo"

        del obj.coupon
        with pytest.raises(AttributeError):
            obj.coupon

        obj.refresh_from({"coupon": "foo"}, api_key="bar", partial=True)
        assert obj.coupon == "foo"

    def test_deletion_metadata(self):
        obj = StripeObject.construct_from(
            {"metadata": {"key": "value"}}, "mykey"
        )

        assert obj.metadata["key"] == "value"

        del obj.metadata["key"]
        with pytest.raises(KeyError):
            obj.metadata["key"]

    def test_copy(self):
        nested = StripeObject.construct_from({"value": "bar"}, "mykey")
        obj = StripeObject.construct_from(
            {"empty": "", "value": "foo", "nested": nested},
            "mykey",
            stripe_account="myaccount",
        )

        copied = copy(obj)

        assert copied.empty == ""
        assert copied.value == "foo"
        assert copied.nested.value == "bar"

        assert copied.api_key == "mykey"
        assert copied.stripe_account == "myaccount"

        # Verify that we're not deep copying nested values.
        assert id(nested) == id(copied.nested)

    def test_deepcopy(self):
        nested = StripeObject.construct_from({"value": "bar"}, "mykey")
        obj = StripeObject.construct_from(
            {"empty": "", "value": "foo", "nested": nested},
            "mykey",
            stripe_account="myaccount",
        )

        copied = deepcopy(obj)

        assert copied.empty == ""
        assert copied.value == "foo"
        assert copied.nested.value == "bar"

        assert copied.api_key == "mykey"
        assert copied.stripe_account == "myaccount"

        # Verify that we're actually deep copying nested values.
        assert id(nested) != id(copied.nested)

    def test_to_dict_recursive(self):
        foo = StripeObject.construct_from({"value": "foo"}, "mykey")
        bar = StripeObject.construct_from({"value": "bar"}, "mykey")
        obj = StripeObject.construct_from(
            {"empty": "", "value": "foobar", "nested": [foo, bar]}, "mykey"
        )

        d = obj._to_dict_recursive()
        assert d == {
            "empty": "",
            "value": "foobar",
            "nested": [{"value": "foo"}, {"value": "bar"}],
        }
        assert not isinstance(d["nested"][0], StripeObject)
        assert not isinstance(d["nested"][1], StripeObject)

    def test_serialize_empty_string_unsets(self):
        class SerializeToEmptyString(StripeObject):
            def serialize(self, previous):
                return ""

        nested = SerializeToEmptyString.construct_from(
            {"value": "bar"}, "mykey"
        )
        obj = StripeObject.construct_from({"nested": nested}, "mykey")

        assert obj.serialize(None) == {"nested": ""}

    def test_serialize_empty_when_nothing_changed(self):
        obj = StripeObject.construct_from({"id": "x", "name": "alice"}, "key")
        assert obj.serialize(None) == {}

    def test_serialize_includes_unsaved_values(self):
        obj = StripeObject.construct_from({"id": "x", "name": "alice"}, "key")
        obj.name = "bob"
        assert obj.serialize(None) == {"name": "bob"}

    def test_serialize_multiple_unsaved_values(self):
        obj = StripeObject.construct_from(
            {"id": "x", "name": "alice", "email": "a@example.com"}, "key"
        )
        obj.name = "bob"
        obj.email = "b@example.com"
        result = obj.serialize(None)
        assert result == {"name": "bob", "email": "b@example.com"}

    def test_serialize_setting_same_value_still_serializes(self):
        obj = StripeObject.construct_from({"id": "x", "name": "alice"}, "key")
        obj.name = "alice"
        assert obj.serialize(None) == {"name": "alice"}

    def test_serialize_skips_id(self):
        obj = StripeObject.construct_from({"id": "x"}, "key")
        obj._unsaved_values.add("id")
        assert obj.serialize(None) == {}

    def test_serialize_skips_underscore_prefixed_keys(self):
        obj = StripeObject.construct_from({"id": "x", "_secret": "s"}, "key")
        obj._unsaved_values.add("_secret")
        assert obj.serialize(None) == {}

    def test_serialize_skips_api_resources(self):
        inner = stripe.Customer.construct_from(
            {"id": "cus_123", "object": "customer"}, "key"
        )
        obj = StripeObject.construct_from(
            {"id": "x", "customer": inner}, "key"
        )
        obj._unsaved_values.add("customer")
        assert obj.serialize(None) == {}

    def test_serialize_nested_changed_object(self):
        obj = StripeObject.construct_from(
            {"id": "x", "metadata": {"key1": "val1"}}, "key"
        )
        obj.metadata["key1"] = "val2"
        result = obj.serialize(None)
        assert result == {"metadata": {"key1": "val2"}}

    def test_serialize_nested_unchanged_excluded(self):
        obj = StripeObject.construct_from(
            {"id": "x", "metadata": {"key1": "val1"}}, "key"
        )
        assert obj.serialize(None) == {}

    def test_serialize_nested_new_key(self):
        obj = StripeObject.construct_from(
            {"id": "x", "metadata": {"key1": "val1"}}, "key"
        )
        obj.metadata["key2"] = "val2"
        result = obj.serialize(None)
        assert result == {"metadata": {"key2": "val2"}}

    def test_serialize_deeply_nested(self):
        obj = StripeObject.construct_from(
            {
                "id": "x",
                "level1": {
                    "level2": {"deep": "original"},
                },
            },
            "key",
        )
        obj.level1.level2.deep = "changed"
        result = obj.serialize(None)
        assert result == {"level1": {"level2": {"deep": "changed"}}}

    def test_serialize_diff_removed_key_becomes_empty_string(self):
        obj = StripeObject.construct_from(
            {"id": "x", "metadata": {"key1": "v1", "key2": "v2"}}, "key"
        )
        obj.metadata = {"key1": "v1"}
        result = obj.serialize(None)
        assert result == {"metadata": {"key1": "v1", "key2": ""}}

    def test_serialize_unsaved_none_becomes_empty_string(self):
        obj = StripeObject.construct_from({"id": "x", "name": "alice"}, "key")
        obj.name = None
        assert obj.serialize(None) == {"name": ""}

    def test_serialize_unsaved_empty_string(self):
        # __setitem__ rejects empty strings, so construct with it
        # already empty and mark it as unsaved
        obj = StripeObject.construct_from({"id": "x", "name": ""}, "key")
        obj._unsaved_values.add("name")
        assert obj.serialize(None) == {"name": ""}

    def test_serialize_explicit_previous_used_for_diff(self):
        obj = StripeObject.construct_from({"id": "x", "metadata": {}}, "key")
        obj.metadata = {"key1": "new"}
        previous = {"metadata": {"key1": "old", "removed_key": "gone"}}
        result = obj.serialize(previous)
        assert result == {"metadata": {"key1": "new", "removed_key": ""}}

    def test_serialize_previous_none_uses_internal_previous(self):
        obj = StripeObject.construct_from(
            {"id": "x", "metadata": {"key1": "v1", "key2": "v2"}}, "key"
        )
        obj.metadata = {"key1": "v1"}
        result = obj.serialize(None)
        assert result == {"metadata": {"key1": "v1", "key2": ""}}

    def test_serialize_additional_owners(self):
        obj = StripeObject.construct_from(
            {
                "id": "x",
                "additional_owners": [
                    {"first_name": "alice"},
                    {"first_name": "bob"},
                ],
            },
            "key",
        )
        result = obj.serialize(None)
        # additional_owners is always serialized when non-None;
        # child objects have no unsaved values so they serialize to {}
        assert "additional_owners" in result
        assert result["additional_owners"]["0"] == {}
        assert result["additional_owners"]["1"] == {}

    def test_serialize_additional_owners_none_excluded(self):
        obj = StripeObject.construct_from(
            {"id": "x", "additional_owners": None}, "key"
        )
        assert obj.serialize(None) == {}

    def test_serialize_after_construct_and_modify(self):
        obj = StripeObject.construct_from(
            {
                "id": "obj_123",
                "name": "original",
                "metadata": {"env": "test"},
            },
            "key",
        )
        obj.name = "updated"
        obj.metadata["env"] = "prod"
        result = obj.serialize(None)
        assert result == {
            "name": "updated",
            "metadata": {"env": "prod"},
        }

    def test_serialize_after_refresh_resets_unsaved(self):
        obj = StripeObject.construct_from(
            {"id": "x", "name": "original"}, "key"
        )
        obj.name = "changed"
        assert obj.serialize(None) == {"name": "changed"}

        obj.refresh_from({"id": "x", "name": "from_api"}, "key")
        assert obj.serialize(None) == {}

    def test_serialize_after_refresh_then_modify(self):
        obj = StripeObject.construct_from(
            {"id": "x", "name": "v1", "email": "a@example.com"}, "key"
        )
        obj.refresh_from(
            {"id": "x", "name": "v2", "email": "a@example.com"}, "key"
        )
        obj.email = "new@example.com"
        result = obj.serialize(None)
        assert result == {"email": "new@example.com"}

    def test_serialize_new_key(self):
        obj = StripeObject.construct_from({"id": "x"}, "key")
        obj["new_field"] = "value"
        assert obj.serialize(None) == {"new_field": "value"}

    def test_serialize_new_key_via_attribute(self):
        obj = StripeObject.construct_from({"id": "x"}, "key")
        obj.new_field = "value"
        assert obj.serialize(None) == {"new_field": "value"}

    def test_field_name_remapping(self):
        class Foo(StripeObject):
            _field_remappings = {"getter_name": "data_name"}

        obj = Foo.construct_from({"data_name": "foo"}, "mykey")
        assert obj.getter_name == "foo"

    def test_sends_request_with_api_key(self, http_client_mock):
        obj = StripeObject("id", "key")

        http_client_mock.stub_request(
            "get",
            path="/foo",
        )

        obj._request("get", "/foo", base_address="api")

        http_client_mock.assert_requested(
            api_key="key",
            stripe_account=None,
        )

    @pytest.mark.anyio
    async def test_request_async_succeeds(self, http_client_mock):
        http_client_mock.stub_request("get", "/foo")
        obj = StripeObject("id", "key")
        await obj._request_async("get", "/foo", base_address="api")
        http_client_mock.assert_requested(
            api_key="key",
            stripe_account=None,
        )

    def test_refresh_from_creates_new_requestor(self):
        obj = StripeObject.construct_from({}, key="origkey")

        orig_requestor = obj._requestor
        assert obj.api_key == "origkey"

        obj.refresh_from({}, "newkey")

        new_requestor = obj._requestor
        assert orig_requestor is not new_requestor
        assert obj.api_key == "newkey"
        assert orig_requestor.api_key == "origkey"

    def test_can_update_api_key(self, http_client_mock):
        obj = StripeObject("id", "key")

        http_client_mock.stub_request(
            "get",
            path="/foo",
        )

        obj.api_key = "key2"
        obj._request("get", "/foo", base_address="api")

        assert "api_key" not in obj.to_dict()

        http_client_mock.assert_requested(
            api_key="key2",
            stripe_account=None,
        )

    def test_invoice_payment_method_gets_special_error(self):
        def is_good_error(e: Exception) -> bool:
            return "multiple-partial-payments-on-invoices" in str(e)

        i = Invoice()

        with pytest.raises(AttributeError) as e:
            i.payment_intent  # type: ignore
        assert is_good_error(e.value)

        with pytest.raises(KeyError) as e:
            i["payment_intent"]
        assert is_good_error(e.value)

        # only that property gets the special error
        with pytest.raises(AttributeError) as e:
            i.blah  # type: ignore
        assert not is_good_error(e.value)

        with pytest.raises(KeyError) as e:
            i["blah"]
        assert not is_good_error(e.value)

        # other classes don't have that special error
        so = StripeObject()
        with pytest.raises(AttributeError) as e:
            so.payment_intent  # type: ignore
        assert not is_good_error(e.value)

        with pytest.raises(KeyError) as e:
            so["payment_intent"]
        assert not is_good_error(e.value)

    def test_eq_same_data(self):
        a = StripeObject.construct_from({"id": "x", "name": "a"}, "key")
        b = StripeObject.construct_from({"id": "x", "name": "a"}, "key")
        assert a == b

    def test_eq_different_data(self):
        a = StripeObject.construct_from({"id": "x", "name": "a"}, "key")
        b = StripeObject.construct_from({"id": "x", "name": "b"}, "key")
        assert a != b

    def test_eq_different_types_not_equal(self):
        data = {"id": "x", "name": "a"}
        invoice = stripe.Invoice.construct_from(
            {**data, "object": "invoice"}, "key"
        )
        customer = stripe.Customer.construct_from(
            {**data, "object": "customer"}, "key"
        )
        assert invoice != customer

    def test_eq_same_resource_type(self):
        a = stripe.Customer.construct_from(
            {"id": "cus_1", "object": "customer", "name": "alice"}, "key"
        )
        b = stripe.Customer.construct_from(
            {"id": "cus_1", "object": "customer", "name": "alice"}, "key"
        )
        assert a == b

    def test_eq_diff_resource_type_same_data(self):
        a = stripe.Customer.construct_from(
            {"id": "cus_1", "object": "customer", "name": "alice"}, "key"
        )
        b = stripe.Invoice.construct_from(
            {"id": "cus_1", "object": "customer", "name": "alice"}, "key"
        )
        assert a != b

    def test_eq_not_equal_to_dict(self):
        obj = StripeObject.construct_from({"id": "x"}, "key")
        assert obj != {"id": "x"}

    def test_is_not_dict(self):
        obj = StripeObject("id", "key")
        assert not isinstance(obj, dict)

    def test_items_field_not_shadowed_by_dict_items(self):
        obj = StripeObject.construct_from(
            {
                "id": "sub_123",
                "object": "subscription",
                "items": {"object": "list", "data": [{"id": "si_123"}]},
            },
            "key",
        )
        assert isinstance(obj.items, stripe.ListObject)

    def test_to_dict(self):
        obj = StripeObject.construct_from(
            {"id": "foo", "name": "bar"},
            "key",
        )
        d = obj.to_dict()
        assert d == {"id": "foo", "name": "bar"}
        assert isinstance(d, dict)
        assert not isinstance(d, StripeObject)

    def test_to_dict_recursive_by_default(self):
        inner = StripeObject.construct_from({"nested": "val"}, "key")
        obj = StripeObject.construct_from({"id": "x", "child": inner}, "key")
        d = obj.to_dict()
        assert d == {"id": "x", "child": {"nested": "val"}}
        assert isinstance(d["child"], dict)
        assert not isinstance(d["child"], StripeObject)

    def test_to_dict_non_recursive(self):
        inner = StripeObject.construct_from({"nested": "val"}, "key")
        obj = StripeObject.construct_from({"id": "x", "child": inner}, "key")
        d = obj.to_dict(recursive=False)
        assert d["id"] == "x"
        # non-recursive preserves nested StripeObjects
        assert isinstance(d["child"], StripeObject)

    def test_to_dict_is_a_copy(self):
        obj = StripeObject.construct_from({"id": "x", "name": "a"}, "key")
        d = obj.to_dict()
        d["name"] = "mutated"
        assert obj.name == "a"

    def test_to_dict_with_list_of_nested_objects(self):
        obj = StripeObject.construct_from(
            {"id": "x", "items": [{"a": 1}, {"b": 2}]}, "key"
        )
        d = obj.to_dict()
        assert d == {"id": "x", "items": [{"a": 1}, {"b": 2}]}
        assert not isinstance(d["items"][0], StripeObject)

    def test_to_dict_json_serializable_converts_decimal(self):
        obj = StripeObject.construct_from(
            {"amount": Decimal("9.99"), "name": "foo"}, "key"
        )
        d = obj.to_dict(for_json=True)
        assert d == {"amount": "9.99", "name": "foo"}
        assert isinstance(d["amount"], str)

    def test_to_dict_json_serializable_converts_datetime(self):
        dt = datetime.datetime(
            2024, 1, 15, 12, 0, 0, tzinfo=datetime.timezone.utc
        )
        obj = StripeObject.construct_from({"created": dt, "id": "x"}, "key")
        d = obj.to_dict(for_json=True)
        assert isinstance(d["created"], int)

    def test_to_dict_json_serializable_nested(self):
        inner = StripeObject.construct_from({"amount": Decimal("1.23")}, "key")
        obj = StripeObject.construct_from({"child": inner, "id": "x"}, "key")
        d = obj.to_dict(for_json=True)
        assert d["child"] == {"amount": "1.23"}
        assert isinstance(d["child"]["amount"], str)

    def test_to_dict_json_serializable_false_preserves_decimal(self):
        obj = StripeObject.construct_from({"amount": Decimal("9.99")}, "key")
        d = obj.to_dict()
        assert isinstance(d["amount"], Decimal)

    def test_update_sets_values(self):
        obj = StripeObject.construct_from({"id": "x", "name": "a"}, "key")
        obj.update({"name": "b", "email": "b@example.com"})
        assert obj.name == "b"
        assert obj.email == "b@example.com"

    def test_update_marks_keys_unsaved(self):
        obj = StripeObject.construct_from({"id": "x", "name": "a"}, "key")
        obj.update({"name": "b", "email": "b@example.com"})
        assert "name" in obj._unsaved_values
        assert "email" in obj._unsaved_values

    def test_update_shows_in_serialize(self):
        obj = StripeObject.construct_from({"id": "x", "name": "a"}, "key")
        obj.update({"name": "b"})
        assert obj.serialize(None) == {"name": "b"}

    def test_update_multiple_calls(self):
        obj = StripeObject.construct_from({"id": "x"}, "key")
        obj.update({"a": 1})
        obj.update({"b": 2})
        assert obj.a == 1
        assert obj.b == 2
        assert obj.serialize(None) == {"a": 1, "b": 2}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe._source_transaction import SourceTransaction
    from stripe.params._source_transaction_list_params import (
        SourceTransactionListParams,
    )


class SourceTransactionService(StripeService):
    def list(
        self,
        source: str,
        params: Optional["SourceTransactionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SourceTransaction]":
        """
        List source transactions for a given source.
        """
        return cast(
            "ListObject[SourceTransaction]",
            self._request(
                "get",
                "/v1/sources/{source}/source_transactions".format(
                    source=sanitize_id(source),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        source: str,
        params: Optional["SourceTransactionListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[SourceTransaction]":
        """
        List source transactions for a given source.
        """
        return cast(
            "ListObject[SourceTransaction]",
            await self._request_async(
                "get",
                "/v1/sources/{source}/source_transactions".format(
                    source=sanitize_id(source),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class AlertListParams(TypedDict):
    created: NotRequired[str]
    """
    Filter for alerts created at the specified timestamp.
    """
    created_gt: NotRequired[str]
    """
    Filter for alerts created after the specified timestamp.
    """
    created_gte: NotRequired[str]
    """
    Filter for alerts created on or after the specified timestamp.
    """
    created_lt: NotRequired[str]
    """
    Filter for alerts created before the specified timestamp.
    """
    created_lte: NotRequired[str]
    """
    Filter for alerts created on or before the specified timestamp.
    """
    limit: NotRequired[int]
    """
    The page limit.
    """
    severity: NotRequired[Literal["critical", "non_critical"]]
    """
    Filter by alert severity.
    """
    status: NotRequired[Literal["open", "resolved"]]
    """
    Filter by alert status.
    """
    types: NotRequired[
        List[
            Literal[
                "api_error",
                "api_latency",
                "authorization_rate_drop",
                "elements_error",
                "event_generation_failure",
                "fraud_rate",
                "invoice_count_dropped",
                "issuing_authorization_request_errors",
                "issuing_authorization_request_timeout",
                "meter_event_summaries_delayed",
                "metronome_notification_latency",
                "payment_method_error",
                "sepa_debit_delayed",
                "traffic_volume_drop",
                "webhook_latency",
            ]
        ]
    ]
    """
    Filter by alert types.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._nested_resource_class_methods import nested_resource_class_methods
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._credit_note_line_item import CreditNoteLineItem
    from stripe._customer import Customer
    from stripe._customer_balance_transaction import CustomerBalanceTransaction
    from stripe._discount import Discount
    from stripe._invoice import Invoice
    from stripe._refund import Refund as RefundResource
    from stripe._shipping_rate import ShippingRate
    from stripe._tax_rate import TaxRate
    from stripe.billing._credit_balance_transaction import (
        CreditBalanceTransaction,
    )
    from stripe.params._credit_note_create_params import CreditNoteCreateParams
    from stripe.params._credit_note_list_lines_params import (
        CreditNoteListLinesParams,
    )
    from stripe.params._credit_note_list_params import CreditNoteListParams
    from stripe.params._credit_note_modify_params import CreditNoteModifyParams
    from stripe.params._credit_note_preview_lines_params import (
        CreditNotePreviewLinesParams,
    )
    from stripe.params._credit_note_preview_params import (
        CreditNotePreviewParams,
    )
    from stripe.params._credit_note_retrieve_params import (
        CreditNoteRetrieveParams,
    )
    from stripe.params._credit_note_void_credit_note_params import (
        CreditNoteVoidCreditNoteParams,
    )


@nested_resource_class_methods("line")
class CreditNote(
    CreateableAPIResource["CreditNote"],
    ListableAPIResource["CreditNote"],
    UpdateableAPIResource["CreditNote"],
):
    """
    Issue a credit note to adjust an invoice's amount after the invoice is finalized.

    Related guide: [Credit notes](https://docs.stripe.com/billing/invoices/credit-notes)
    """

    OBJECT_NAME: ClassVar[Literal["credit_note"]] = "credit_note"

    class DiscountAmount(StripeObject):
        amount: int
        """
        The amount, in cents (or local equivalent), of the discount.
        """
        discount: ExpandableField["Discount"]
        """
        The discount that was applied to get this discount amount.
        """

    class PretaxCreditAmount(StripeObject):
        amount: int
        """
        The amount, in cents (or local equivalent), of the pretax credit amount.
        """
        credit_balance_transaction: Optional[
            ExpandableField["CreditBalanceTransaction"]
        ]
        """
        The credit balance transaction that was applied to get this pretax credit amount.
        """
        discount: Optional[ExpandableField["Discount"]]
        """
        The discount that was applied to get this pretax credit amount.
        """
        type: Literal["credit_balance_transaction", "discount"]
        """
        Type of the pretax credit amount referenced.
        """

    class Refund(StripeObject):
        class PaymentRecordRefund(StripeObject):
            payment_record: str
            """
            ID of the payment record.
            """
            refund_group: str
            """
            ID of the refund group.
            """

        amount_refunded: int
        """
        Amount of the refund that applies to this credit note, in cents (or local equivalent).
        """
        payment_record_refund: Optional[PaymentRecordRefund]
        """
        The PaymentRecord refund details associated with this credit note refund.
        """
        refund: ExpandableField["RefundResource"]
        """
        ID of the refund.
        """
        type: Optional[Literal["payment_record_refund", "refund"]]
        """
        Type of the refund, one of `refund` or `payment_record_refund`.
        """
        _inner_class_types = {"payment_record_refund": PaymentRecordRefund}

    class ShippingCost(StripeObject):
        class Tax(StripeObject):
            amount: int
            """
            Amount of tax applied for this rate.
            """
            rate: "TaxRate"
            """
            Tax rates can be applied to [invoices](https://docs.stripe.com/invoicing/taxes/tax-rates), [subscriptions](https://docs.stripe.com/billing/taxes/tax-rates) and [Checkout Sessions](https://docs.stripe.com/payments/checkout/use-manual-tax-rates) to collect tax.

            Related guide: [Tax rates](https://docs.stripe.com/billing/taxes/tax-rates)
            """
            taxability_reason: Optional[
                Literal[
                    "customer_exempt",
                    "not_collecting",
                    "not_subject_to_tax",
                    "not_supported",
                    "portion_product_exempt",
                    "portion_reduced_rated",
                    "portion_standard_rated",
                    "product_exempt",
                    "product_exempt_holiday",
                    "proportionally_rated",
                    "reduced_rated",
                    "reverse_charge",
                    "standard_rated",
                    "taxable_basis_reduced",
                    "zero_rated",
                ]
            ]
            """
            The reasoning behind this tax, for example, if the product is tax exempt. The possible values for this field may be extended as new tax rules are supported.
            """
            taxable_amount: Optional[int]
            """
            The amount on which tax is calculated, in cents (or local equivalent).
            """

        amount_subtotal: int
        """
        Total shipping cost before any taxes are applied.
        """
        amount_tax: int
        """
        Total tax amount applied due to shipping costs. If no tax was applied, defaults to 0.
        """
        amount_total: int
        """
        Total shipping cost after taxes are applied.
        """
        shipping_rate: Optional[ExpandableField["ShippingRate"]]
        """
        The ID of the ShippingRate for this invoice.
        """
        taxes: Optional[List[Tax]]
        """
        The taxes applied to the shipping rate.
        """
        _inner_class_types = {"taxes": Tax}

    class TotalTax(StripeObject):
        class TaxRateDetails(StripeObject):
            tax_rate: ExpandableField["TaxRate"]
            """
            ID of the tax rate
            """

        amount: int
        """
        The amount of the tax, in cents (or local equivalent).
        """
        tax_behavior: Literal["exclusive", "inclusive"]
        """
        Whether this tax is inclusive or exclusive.
        """
        tax_rate_details: Optional[TaxRateDetails]
        """
        Additional details about the tax rate. Only present when `type` is `tax_rate_details`.
        """
        taxability_reason: Literal[
            "customer_exempt",
            "not_available",
            "not_collecting",
            "not_subject_to_tax",
            "not_supported",
            "portion_product_exempt",
            "portion_reduced_rated",
            "portion_standard_rated",
            "product_exempt",
            "product_exempt_holiday",
            "proportionally_rated",
            "reduced_rated",
            "reverse_charge",
            "standard_rated",
            "taxable_basis_reduced",
            "zero_rated",
        ]
        """
        The reasoning behind this tax, for example, if the product is tax exempt. The possible values for this field may be extended as new tax rules are supported.
        """
        taxable_amount: Optional[int]
        """
        The amount on which tax is calculated, in cents (or local equivalent).
        """
        type: Literal["tax_rate_details"]
        """
        The type of tax information.
        """
        _inner_class_types = {"tax_rate_details": TaxRateDetails}

    amount: int
    """
    The integer amount in cents (or local equivalent) representing the total amount of the credit note, including tax.
    """
    amount_shipping: int
    """
    This is the sum of all the shipping amounts.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    customer: ExpandableField["Customer"]
    """
    ID of the customer.
    """
    customer_account: Optional[str]
    """
    ID of the account representing the customer.
    """
    customer_balance_transaction: Optional[
        ExpandableField["CustomerBalanceTransaction"]
    ]
    """
    Customer balance transaction related to this credit note.
    """
    discount_amount: int
    """
    The integer amount in cents (or local equivalent) representing the total amount of discount that was credited.
    """
    discount_amounts: List[DiscountAmount]
    """
    The aggregate amounts calculated per discount for all line items.
    """
    effective_at: Optional[int]
    """
    The date when this credit note is in effect. Same as `created` unless overwritten. When defined, this value replaces the system-generated 'Date of issue' printed on the credit note PDF.
    """
    id: str
    """
    Unique identifier for the object.
    """
    invoice: ExpandableField["Invoice"]
    """
    ID of the invoice.
    """
    lines: ListObject["CreditNoteLineItem"]
    """
    Line items that make up the credit note
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    memo: Optional[str]
    """
    Customer-facing text that appears on the credit note PDF.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    number: str
    """
    A unique number that identifies this particular credit note and appears on the PDF of the credit note and its associated invoice.
    """
    object: Literal["credit_note"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    out_of_band_amount: Optional[int]
    """
    Amount that was credited outside of Stripe.
    """
    pdf: str
    """
    The link to download the PDF of the credit note.
    """
    post_payment_amount: int
    """
    The amount of the credit note that was refunded to the customer, credited to the customer's balance, credited outside of Stripe, or any combination thereof.
    """
    pre_payment_amount: int
    """
    The amount of the credit note by which the invoice's `amount_remaining` and `amount_due` were reduced.
    """
    pretax_credit_amounts: List[PretaxCreditAmount]
    """
    The pretax credit amounts (ex: discount, credit grants, etc) for all line items.
    """
    reason: Optional[
        Literal[
            "duplicate", "fraudulent", "order_change", "product_unsatisfactory"
        ]
    ]
    """
    Reason for issuing this credit note, one of `duplicate`, `fraudulent`, `order_change`, or `product_unsatisfactory`
    """
    refunds: List[Refund]
    """
    Refunds related to this credit note.
    """
    shipping_cost: Optional[ShippingCost]
    """
    The details of the cost of shipping, including the ShippingRate applied to the invoice.
    """
    status: Literal["issued", "void"]
    """
    Status of this credit note, one of `issued` or `void`. Learn more about [voiding credit notes](https://docs.stripe.com/billing/invoices/credit-notes#voiding).
    """
    subtotal: int
    """
    The integer amount in cents (or local equivalent) representing the amount of the credit note, excluding exclusive tax and invoice level discounts.
    """
    subtotal_excluding_tax: Optional[int]
    """
    The integer amount in cents (or local equivalent) representing the amount of the credit note, excluding all tax and invoice level discounts.
    """
    total: int
    """
    The integer amount in cents (or local equivalent) representing the total amount of the credit note, including tax and all discount.
    """
    total_excluding_tax: Optional[int]
    """
    The integer amount in cents (or local equivalent) representing the total amount of the credit note, excluding tax, but including discounts.
    """
    total_taxes: Optional[List[TotalTax]]
    """
    The aggregate tax information for all line items.
    """
    type: Literal["mixed", "post_payment", "pre_payment"]
    """
    Type of this credit note, one of `pre_payment` or `post_payment`. A `pre_payment` credit note means it was issued when the invoice was open. A `post_payment` credit note means it was issued when the invoice was paid.
    """
    voided_at: Optional[int]
    """
    The time that the credit note was voided.
    """

    @classmethod
    def create(
        cls, **params: Unpack["CreditNoteCreateParams"]
    ) -> "CreditNote":
        """
        Issue a credit note to adjust the amount of a finalized invoice. A credit note will first reduce the invoice's amount_remaining (and amount_due), but not below zero.
        This amount is indicated by the credit note's pre_payment_amount. The excess amount is indicated by post_payment_amount, and it can result in any combination of the following:


        Refunds: create a new refund (using refund_amount) or link existing refunds (using refunds).
        Customer balance credit: credit the customer's balance (using credit_amount) which will be automatically applied to their next invoice when it's finalized.
        Outside of Stripe credit: record the amount that is or will be credited outside of Stripe (using out_of_band_amount).


        The sum of refunds, customer balance credits, and outside of Stripe credits must equal the post_payment_amount.

        You may issue multiple credit notes for an invoice. Each credit note may increment the invoice's pre_payment_credit_notes_amount,
        post_payment_credit_notes_amount, or both, depending on the invoice's amount_remaining at the time of credit note creation.

        For invoices that also have refunds created through the [Refund API](https://docs.stripe.com/docs/api/refunds), the credit note API subtracts those refund amounts from the maximum creditable amount. This prevents the combined credit notes and refunds from exceeding the invoice amount. If you use both, ensure the combined total does not exceed the invoice's paid amount.
        """
        return cast(
            "CreditNote",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["CreditNoteCreateParams"]
    ) -> "CreditNote":
        """
        Issue a credit note to adjust the amount of a finalized invoice. A credit note will first reduce the invoice's amount_remaining (and amount_due), but not below zero.
        This amount is indicated by the credit note's pre_payment_amount. The excess amount is indicated by post_payment_amount, and it can result in any combination of the following:


        Refunds: create a new refund (using refund_amount) or link existing refunds (using refunds).
        Customer balance credit: credit the customer's balance (using credit_amount) which will be automatically applied to their next invoice when it's finalized.
        Outside of Stripe credit: record the amount that is or will be credited outside of Stripe (using out_of_band_amount).


        The sum of refunds, customer balance credits, and outside of Stripe credits must equal the post_payment_amount.

        You may issue multiple credit notes for an invoice. Each credit note may increment the invoice's pre_payment_credit_notes_amount,
        post_payment_credit_notes_amount, or both, depending on the invoice's amount_remaining at the time of credit note creation.

        For invoices that also have refunds created through the [Refund API](https://docs.stripe.com/docs/api/refunds), the credit note API subtracts those refund amounts from the maximum creditable amount. This prevents the combined credit notes and refunds from exceeding the invoice amount. If you use both, ensure the combined total does not exceed the invoice's paid amount.
        """
        return cast(
            "CreditNote",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["CreditNoteListParams"]
    ) -> ListObject["CreditNote"]:
        """
        Returns a list of credit notes.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["CreditNoteListParams"]
    ) -> ListObject["CreditNote"]:
        """
        Returns a list of credit notes.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["CreditNoteModifyParams"]
    ) -> "CreditNote":
        """
        Updates an existing credit note.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "CreditNote",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["CreditNoteModifyParams"]
    ) -> "CreditNote":
        """
        Updates an existing credit note.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "CreditNote",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def preview(
        cls, **params: Unpack["CreditNotePreviewParams"]
    ) -> "CreditNote":
        """
        Get a preview of a credit note without creating it.
        """
        return cast(
            "CreditNote",
            cls._static_request(
                "get",
                "/v1/credit_notes/preview",
                params=params,
            ),
        )

    @classmethod
    async def preview_async(
        cls, **params: Unpack["CreditNotePreviewParams"]
    ) -> "CreditNote":
        """
        Get a preview of a credit note without creating it.
        """
        return cast(
            "CreditNote",
            await cls._static_request_async(
                "get",
                "/v1/credit_notes/preview",
                params=params,
            ),
        )

    @classmethod
    def preview_lines(
        cls, **params: Unpack["CreditNotePreviewLinesParams"]
    ) -> ListObject["CreditNoteLineItem"]:
        """
        When retrieving a credit note preview, you'll get a lines property containing the first handful of those items. This URL you can retrieve the full (paginated) list of line items.
        """
        return cast(
            ListObject["CreditNoteLineItem"],
            cls._static_request(
                "get",
                "/v1/credit_notes/preview/lines",
                params=params,
            ),
        )

    @classmethod
    async def preview_lines_async(
        cls, **params: Unpack["CreditNotePreviewLinesParams"]
    ) -> ListObject["CreditNoteLineItem"]:
        """
        When retrieving a credit note preview, you'll get a lines property containing the first handful of those items. This URL you can retrieve the full (paginated) list of line items.
        """
        return cast(
            ListObject["CreditNoteLineItem"],
            await cls._static_request_async(
                "get",
                "/v1/credit_notes/preview/lines",
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["CreditNoteRetrieveParams"]
    ) -> "CreditNote":
        """
        Retrieves the credit note object with the given identifier.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["CreditNoteRetrieveParams"]
    ) -> "CreditNote":
        """
        Retrieves the credit note object with the given identifier.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_void_credit_note(
        cls, id: str, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        return cast(
            "CreditNote",
            cls._static_request(
                "post",
                "/v1/credit_notes/{id}/void".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def void_credit_note(
        id: str, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        ...

    @overload
    def void_credit_note(
        self, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        ...

    @class_method_variant("_cls_void_credit_note")
    def void_credit_note(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        return cast(
            "CreditNote",
            self._request(
                "post",
                "/v1/credit_notes/{id}/void".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_void_credit_note_async(
        cls, id: str, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        return cast(
            "CreditNote",
            await cls._static_request_async(
                "post",
                "/v1/credit_notes/{id}/void".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def void_credit_note_async(
        id: str, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        ...

    @overload
    async def void_credit_note_async(
        self, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        ...

    @class_method_variant("_cls_void_credit_note_async")
    async def void_credit_note_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CreditNoteVoidCreditNoteParams"]
    ) -> "CreditNote":
        """
        Marks a credit note as void. Learn more about [voiding credit notes](https://docs.stripe.com/docs/billing/invoices/credit-notes#voiding).
        """
        return cast(
            "CreditNote",
            await self._request_async(
                "post",
                "/v1/credit_notes/{id}/void".format(
                    id=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def list_lines(
        cls, credit_note: str, **params: Unpack["CreditNoteListLinesParams"]
    ) -> ListObject["CreditNoteLineItem"]:
        """
        When retrieving a credit note, you'll get a lines property containing the first handful of those items. There is also a URL where you can retrieve the full (paginated) list of line items.
        """
        return cast(
            ListObject["CreditNoteLineItem"],
            cls._static_request(
                "get",
                "/v1/credit_notes/{credit_note}/lines".format(
                    credit_note=sanitize_id(credit_note)
                ),
                params=params,
            ),
        )

    @classmethod
    async def list_lines_async(
        cls, credit_note: str, **params: Unpack["CreditNoteListLinesParams"]
    ) -> ListObject["CreditNoteLineItem"]:
        """
        When retrieving a credit note, you'll get a lines property containing the first handful of those items. There is also a URL where you can retrieve the full (paginated) list of line items.
        """
        return cast(
            ListObject["CreditNoteLineItem"],
            await cls._static_request_async(
                "get",
                "/v1/credit_notes/{credit_note}/lines".format(
                    credit_note=sanitize_id(credit_note)
                ),
                params=params,
            ),
        )

    _inner_class_types = {
        "discount_amounts": DiscountAmount,
        "pretax_credit_amounts": PretaxCreditAmount,
        "refunds": Refund,
        "shipping_cost": ShippingCost,
        "total_taxes": TotalTax,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class InvoiceItemCreateParams(RequestOptions):
    amount: NotRequired[int]
    """
    The integer amount in cents (or local equivalent) of the charge to be applied to the upcoming invoice. Passing in a negative `amount` will reduce the `amount_due` on the invoice.
    """
    currency: NotRequired[str]
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    customer: NotRequired[str]
    """
    The ID of the customer to bill for this invoice item.
    """
    customer_account: NotRequired[str]
    """
    The ID of the account representing the customer to bill for this invoice item.
    """
    description: NotRequired[str]
    """
    An arbitrary string which you can attach to the invoice item. The description is displayed in the invoice for easy tracking.
    """
    discountable: NotRequired[bool]
    """
    Controls whether discounts apply to this invoice item. Defaults to false for prorations or negative invoice items, and true for all other invoice items.
    """
    discounts: NotRequired["Literal['']|List[InvoiceItemCreateParamsDiscount]"]
    """
    The coupons and promotion codes to redeem into discounts for the invoice item or invoice line item.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    invoice: NotRequired[str]
    """
    The ID of an existing invoice to add this invoice item to. For subscription invoices, when left blank, the invoice item will be added to the next upcoming scheduled invoice. For standalone invoices, the invoice item won't be automatically added unless you pass `pending_invoice_item_behavior: 'include'` when creating the invoice. This is useful when adding invoice items in response to an invoice.created webhook. You can only add invoice items to draft invoices and there is a maximum of 250 items per invoice.
    """
    margins: NotRequired[List[str]]
    """
    The ids of the margins to apply to the invoice item. When set, the `default_margins` on the invoice do not apply to this invoice item.
    """
    metadata: NotRequired[
        "Literal['']|Dict[str, str]|UntypedStripeObject[str]"
    ]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    period: NotRequired["InvoiceItemCreateParamsPeriod"]
    """
    The period associated with this invoice item. When set to different values, the period will be rendered on the invoice. If you have [Stripe Revenue Recognition](https://docs.stripe.com/revenue-recognition) enabled, the period will be used to recognize and defer revenue. See the [Revenue Recognition documentation](https://docs.stripe.com/revenue-recognition/methodology/subscriptions-and-invoicing) for details.
    """
    price_data: NotRequired["InvoiceItemCreateParamsPriceData"]
    """
    Data used to generate a new [Price](https://docs.stripe.com/api/prices) object inline.
    """
    pricing: NotRequired["InvoiceItemCreateParamsPricing"]
    """
    The pricing information for the invoice item.
    """
    quantity: NotRequired[int]
    """
    Non-negative integer. The quantity of units for the invoice item. Use `quantity_decimal` instead to provide decimal precision. This field will be deprecated in favor of `quantity_decimal` in a future version.
    """
    quantity_decimal: NotRequired[Decimal]
    """
    Non-negative decimal with at most 12 decimal places. The quantity of units for the invoice item.
    """
    subscription: NotRequired[str]
    """
    The ID of a subscription to add this invoice item to. When left blank, the invoice item is added to the next upcoming scheduled invoice. When set, scheduled invoices for subscriptions other than the specified subscription will ignore the invoice item. Use this when you want to express that an invoice item has been accrued within the context of a particular subscription.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://docs.stripe.com/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    tax_code: NotRequired["Literal['']|str"]
    """
    A [tax code](https://docs.stripe.com/tax/tax-categories) ID.
    """
    tax_rates: NotRequired[List[str]]
    """
    The tax rates which apply to the invoice item. When set, the `default_tax_rates` on the invoice do not apply to this invoice item.
    """
    unit_amount_decimal: NotRequired[Decimal]
    """
    The decimal unit amount in cents (or local equivalent) of the charge to be applied to the upcoming invoice. This `unit_amount_decimal` will be multiplied by the quantity to get the full amount. Passing in a negative `unit_amount_decimal` will reduce the `amount_due` on the invoice. Accepts at most 12 decimal places.
    """


class InvoiceItemCreateParamsDiscount(TypedDict):
    coupon: NotRequired[str]
    """
    ID of the coupon to create a new discount for.
    """
    discount: NotRequired[str]
    """
    ID of an existing discount on the object (or one of its ancestors) to reuse.
    """
    discount_end: NotRequired["InvoiceItemCreateParamsDiscountDiscountEnd"]
    """
    Details to determine how long the discount should be applied for.
    """
    promotion_code: NotRequired[str]
    """
    ID of the promotion code to create a new discount for.
    """


class InvoiceItemCreateParamsDiscountDiscountEnd(TypedDict):
    duration: NotRequired["InvoiceItemCreateParamsDiscountDiscountEndDuration"]
    """
    Time span for the redeemed discount.
    """
    timestamp: NotRequired[int]
    """
    A precise Unix timestamp for the discount to end. Must be in the future.
    """
    type: Literal["duration", "timestamp"]
    """
    The type of calculation made to determine when the discount ends.
    """


class InvoiceItemCreateParamsDiscountDiscountEndDuration(TypedDict):
    interval: Literal["day", "month", "week", "year"]
    """
    Specifies a type of interval unit. Either `day`, `week`, `month` or `year`.
    """
    interval_count: int
    """
    The number of intervals, as an whole number greater than 0. Stripe multiplies this by the interval type to get the overall duration.
    """


class InvoiceItemCreateParamsPeriod(TypedDict):
    end: int
    """
    The end of the period, which must be greater than or equal to the start. This value is inclusive.
    """
    start: int
    """
    The start of the period. This value is inclusive.
    """


class InvoiceItemCreateParamsPriceData(TypedDict):
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    product: str
    """
    The ID of the [Product](https://docs.stripe.com/api/products) that this [Price](https://docs.stripe.com/api/prices) will belong to.
    """
    tax_behavior: NotRequired[Literal["exclusive", "inclusive", "unspecified"]]
    """
    Only required if a [default tax behavior](https://docs.stripe.com/tax/products-prices-tax-categories-tax-behavior#setting-a-default-tax-behavior-(recommended)) was not provided in the Stripe Tax settings. Specifies whether the price is considered inclusive of taxes or exclusive of taxes. One of `inclusive`, `exclusive`, or `unspecified`. Once specified as either `inclusive` or `exclusive`, it cannot be changed.
    """
    unit_amount: NotRequired[int]
    """
    A positive integer in cents (or local equivalent) (or 0 for a free price) representing how much to charge.
    """
    unit_amount_decimal: NotRequired[Decimal]
    """
    Same as `unit_amount`, but accepts a decimal value in cents (or local equivalent) with at most 12 decimal places. Only one of `unit_amount` and `unit_amount_decimal` can be set.
    """


class InvoiceItemCreateParamsPricing(TypedDict):
    price: NotRequired[str]
    """
    The ID of the price object.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class ApprovalRequest(StripeObject):
    """
    An approval request represents a pending action that requires review before execution.
    """

    OBJECT_NAME: ClassVar[Literal["v2.core.approval_request"]] = (
        "v2.core.approval_request"
    )

    class RequestedBy(StripeObject):
        id: str
        """
        Stripe-defined identifier for the requester (e.g. a restricted API key token).
        """
        name: Optional[str]
        """
        Merchant-defined name for the requester.
        """

    class Review(StripeObject):
        class ReviewedBy(StripeObject):
            id: str
            """
            Stripe-defined identifier for the reviewer (e.g. a restricted API key token).
            """
            name: str
            """
            Merchant-defined name for the reviewer.
            """

        reason: Optional[str]
        """
        The reason provided by the reviewer.
        """
        result: Literal["approved", "rejected"]
        """
        The result of the review.
        """
        reviewed_at: str
        """
        Timestamp when the review was performed.
        """
        reviewed_by: ReviewedBy
        """
        The reviewer who performed the review.
        """
        _inner_class_types = {"reviewed_by": ReviewedBy}

    class Rule(StripeObject):
        name: str
        """
        The name of the rule.
        """

    class StatusDetails(StripeObject):
        class Approved(StripeObject):
            reason: Optional[str]
            """
            The reason provided when approving the request.
            """

        class Canceled(StripeObject):
            pass

        class ExecutionFailed(StripeObject):
            code: str
            """
            The error code for the failed execution.
            """
            message: str
            """
            The error message for the failed execution.
            """
            type: str
            """
            The error type for the failed execution.
            """

        class ExecutionStarted(StripeObject):
            pass

        class ExecutionSucceeded(StripeObject):
            class Result(StripeObject):
                id: str
                """
                The unique identifier of the executed object.
                """
                object: str
                """
                The object type of the executed resource.
                """

            result: Result
            """
            The result of the successful execution.
            """
            _inner_class_types = {"result": Result}

        class Expired(StripeObject):
            pass

        class Failed(StripeObject):
            error_code: str
            """
            The error code for the failed execution.
            """
            error_message: str
            """
            The error message for the failed execution.
            """
            error_type: str
            """
            The error type for the failed execution.
            """

        class Pending(StripeObject):
            pass

        class Rejected(StripeObject):
            reason: Optional[str]
            """
            The reason provided when rejecting the request.
            """

        class Succeeded(StripeObject):
            class Result(StripeObject):
                id: str
                """
                The unique identifier of the executed object.
                """
                object: str
                """
                The object type of the executed resource.
                """

            result: Result
            """
            The result of the successful execution.
            """
            _inner_class_types = {"result": Result}

        approved: Optional[Approved]
        """
        Deprecated: use requires_execution status instead.
        """
        canceled: Optional[Canceled]
        """
        Deprecated: use canceled status instead.
        """
        execution_failed: Optional[ExecutionFailed]
        """
        Deprecated: use failed status instead.
        """
        execution_started: Optional[ExecutionStarted]
        """
        Deprecated: use requires_execution status instead.
        """
        execution_succeeded: Optional[ExecutionSucceeded]
        """
        Deprecated: use succeeded status instead.
        """
        expired: Optional[Expired]
        """
        Deprecated: use expired status instead.
        """
        failed: Optional[Failed]
        """
        Details when the approval request failed.
        """
        pending: Optional[Pending]
        """
        Deprecated: use requires_review status instead.
        """
        rejected: Optional[Rejected]
        """
        Deprecated: use rejected status instead.
        """
        succeeded: Optional[Succeeded]
        """
        Details when the approval request succeeded.
        """
        _inner_class_types = {
            "approved": Approved,
            "canceled": Canceled,
            "execution_failed": ExecutionFailed,
            "execution_started": ExecutionStarted,
            "execution_succeeded": ExecutionSucceeded,
            "expired": Expired,
            "failed": Failed,
            "pending": Pending,
            "rejected": Rejected,
            "succeeded": Succeeded,
        }

    class StatusTransitions(StripeObject):
        canceled_at: Optional[str]
        """
        Timestamp when the approval request was canceled.
        """
        expired_at: Optional[str]
        """
        Timestamp when the approval request expired.
        """
        failed_at: Optional[str]
        """
        Timestamp when the approval request failed.
        """
        rejected_at: Optional[str]
        """
        Timestamp when the approval request was rejected.
        """
        requires_execution_at: Optional[str]
        """
        Timestamp when the approval request moved to requires_execution status.
        """
        succeeded_at: Optional[str]
        """
        Timestamp when the approval request succeeded.
        """

    action: Literal[
        "charge.create",
        "dispute.close",
        "inbound_transfers.money_management.create",
        "invoice.create",
        "outbound_payments.money_management.create",
        "outbound_transfers.money_management.create",
        "payment_intent.create",
        "payment_intent.update",
        "payout.create",
        "price.update",
        "refund.create",
        "setup_intent.create",
        "subscription.create",
        "subscription.update",
        "topup.create",
        "transfer.create",
    ]
    """
    The action that was requested.
    """
    created: str
    """
    Time this ApprovalRequest was created.
    """
    dashboard_url: Optional[str]
    """
    The URL to the dashboard for this ApprovalRequest.
    """
    description: Optional[str]
    """
    A description of the approval request.
    """
    expires_at: str
    """
    The timestamp at which this ApprovalRequest will expire.
    """
    id: str
    """
    The unique identifier for this ApprovalRequest.
    """
    livemode: bool
    """
    Whether this ApprovalRequest is livemode.
    """
    object: Literal["v2.core.approval_request"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    requested_by: RequestedBy
    """
    The requester of this ApprovalRequest.
    """
    review: Optional[Review]
    """
    The review of this ApprovalRequest if it has been reviewed.
    """
    rule: Optional[Rule]
    """
    The rule associated with this ApprovalRequest.
    """
    status: Literal[
        "approved",
        "canceled",
        "execution_failed",
        "execution_started",
        "execution_succeeded",
        "expired",
        "failed",
        "pending",
        "rejected",
        "requires_execution",
        "requires_review",
        "succeeded",
    ]
    """
    The status of this ApprovalRequest.
    """
    status_details: Optional[StatusDetails]
    """
    The details of the status of this ApprovalRequest.
    """
    status_transitions: Optional[StatusTransitions]
    """
    The transitions of the status of this ApprovalRequest.
    """
    _inner_class_types = {
        "requested_by": RequestedBy,
        "review": Review,
        "rule": Rule,
        "status_details": StatusDetails,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.test_helpers import (
        capital as capital,
        issuing as issuing,
        shared_payment as shared_payment,
        terminal as terminal,
        treasury as treasury,
    )
    from stripe.test_helpers._capital_service import (
        CapitalService as CapitalService,
    )
    from stripe.test_helpers._confirmation_token_service import (
        ConfirmationTokenService as ConfirmationTokenService,
    )
    from stripe.test_helpers._customer_service import (
        CustomerService as CustomerService,
    )
    from stripe.test_helpers._issuing_service import (
        IssuingService as IssuingService,
    )
    from stripe.test_helpers._payment_intent_service import (
        PaymentIntentService as PaymentIntentService,
    )
    from stripe.test_helpers._refund_service import (
        RefundService as RefundService,
    )
    from stripe.test_helpers._shared_payment_service import (
        SharedPaymentService as SharedPaymentService,
    )
    from stripe.test_helpers._terminal_service import (
        TerminalService as TerminalService,
    )
    from stripe.test_helpers._test_clock import TestClock as TestClock
    from stripe.test_helpers._test_clock_service import (
        TestClockService as TestClockService,
    )
    from stripe.test_helpers._treasury_service import (
        TreasuryService as TreasuryService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "capital": ("stripe.test_helpers.capital", True),
    "issuing": ("stripe.test_helpers.issuing", True),
    "shared_payment": ("stripe.test_helpers.shared_payment", True),
    "terminal": ("stripe.test_helpers.terminal", True),
    "treasury": ("stripe.test_helpers.treasury", True),
    "CapitalService": ("stripe.test_helpers._capital_service", False),
    "ConfirmationTokenService": (
        "stripe.test_helpers._confirmation_token_service",
        False,
    ),
    "CustomerService": ("stripe.test_helpers._customer_service", False),
    "IssuingService": ("stripe.test_helpers._issuing_service", False),
    "PaymentIntentService": (
        "stripe.test_helpers._payment_intent_service",
        False,
    ),
    "RefundService": ("stripe.test_helpers._refund_service", False),
    "SharedPaymentService": (
        "stripe.test_helpers._shared_payment_service",
        False,
    ),
    "TerminalService": ("stripe.test_helpers._terminal_service", False),
    "TestClock": ("stripe.test_helpers._test_clock", False),
    "TestClockService": ("stripe.test_helpers._test_clock_service", False),
    "TreasuryService": ("stripe.test_helpers._treasury_service", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict
from typing_extensions import Literal, NotRequired, TypedDict


class BatchJobCreateParams(TypedDict):
    endpoint: "BatchJobCreateParamsEndpoint"
    """
    The endpoint configuration for the batch job.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    The metadata of the `batch_job`.
    """
    notification_suppression: NotRequired[
        "BatchJobCreateParamsNotificationSuppression"
    ]
    """
    Notification suppression settings for the batch job.
    """
    skip_validation: bool
    """
    Allows the user to skip validation.
    """


class BatchJobCreateParamsEndpoint(TypedDict):
    http_method: Literal["delete", "post"]
    """
    The HTTP method to use when calling the endpoint.
    """
    path: Literal[
        "/v1/accounts/:account",
        "/v1/accounts",
        "/v1/accounts/:account",
        "/v1/coupons",
        "/v1/coupons/:coupon",
        "/v1/coupons/:coupon",
        "/v1/credit_notes",
        "/v1/customers/:customer",
        "/v1/customers/:customer",
        "/v1/customers",
        "/v1/customers/:customer/discount",
        "/v1/customers/:customer/funding_instructions",
        "/v1/customers/:customer/subscriptions",
        "/v1/customers/:customer/subscriptions",
        "/v1/customers/:customer/subscriptions/:subscription_exposed_id",
        "/v1/customers/:customer/subscriptions/:subscription_exposed_id/discount",
        "/v1/customers/:customer/bank_accounts",
        "/v1/customers/:customer/bank_accounts/:id",
        "/v1/customers/:customer/bank_accounts/:id",
        "/v1/customers/:customer/bank_accounts/:id/verify",
        "/v1/customers/:customer/cards",
        "/v1/customers/:customer/cards/:id",
        "/v1/customers/:customer/cards/:id",
        "/v1/customers/:customer/tax_ids",
        "/v1/customers/:customer/sources",
        "/v1/customers/:customer/sources/:id",
        "/v1/customers/:customer/sources/:id",
        "/v1/customers/:customer/sources/:id/verify",
        "/v1/customers/:customer/balance_transactions",
        "/v1/customers/:customer/balance_transactions/:transaction",
        "/v1/customers/:customer/cash_balance",
        "/v1/customer_sessions",
        "/v1/disputes/:dispute/close",
        "/v1/invoices",
        "/v1/invoices/:invoice",
        "/v1/invoices/:invoice",
        "/v1/invoices/:invoice/pay",
        "/v1/invoices/:invoice/send",
        "/v1/invoices/:invoice/void",
        "/v1/invoices/:invoice/finalize",
        "/v1/invoices/:invoice/mark_uncollectible",
        "/v1/invoices/:invoice/update_lines",
        "/v1/invoices/:invoice/add_lines",
        "/v1/invoices/:invoice/remove_lines",
        "/v1/invoices/create_preview",
        "/v1/invoices/:invoice/lines/:line_item_id",
        "/v1/invoiceitems",
        "/v1/invoiceitems/:invoiceitem",
        "/v1/invoiceitems/:invoiceitem",
        "/v1/invoice_rendering_templates/:template/archive",
        "/v1/invoice_rendering_templates/:template/unarchive",
        "/v1/payment_methods/:payment_method/attach",
        "/v1/prices",
        "/v1/prices/:price",
        "/v1/products",
        "/v1/products/:id",
        "/v1/products/:id",
        "/v1/products/:product/features",
        "/v1/products/:product/features/:id",
        "/v1/promotion_codes",
        "/v1/promotion_codes/:promotion_code",
        "/v1/radar/value_list_items",
        "/v1/refunds",
        "/v1/refunds/:refund/cancel",
        "/v1/subscriptions/:subscription_exposed_id",
        "/v1/subscriptions/:subscription_exposed_id",
        "/v1/subscriptions/:subscription/migrate",
        "/v1/subscriptions",
        "/v1/subscriptions/:subscription/resume",
        "/v1/subscriptions/:subscription/pause",
        "/v1/subscription_items",
        "/v1/subscription_items/:item",
        "/v1/subscription_items/:item",
        "/v1/subscription_schedules",
        "/v1/subscription_schedules/:schedule",
        "/v1/subscription_schedules/:schedule/cancel",
        "/v1/subscription_schedules/:schedule/release",
        "/v1/tax/registrations",
        "/v1/tax/registrations/:id",
        "/v1/tax/settings",
        "/v1/tax/transactions/create_reversal",
        "/v1/tax_ids",
        "/v1/tax_ids/:id",
        "/v1/customers/:customer/tax_ids",
        "/v1/customers/:customer/tax_ids/:id",
        "/v1/tax_rates",
        "/v1/tax_rates/:tax_rate",
    ]
    """
    The path of the endpoint to run this batch job against.
    In the form used in the documentation. For instance, for
    subscription migration this would be `/v1/subscriptions/:id/migrate`.
    """


class BatchJobCreateParamsNotificationSuppression(TypedDict):
    scope: Literal["all", "none"]
    """
    The scope of notification suppression.
    """
